/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products. No
* other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED. TO THE MAXIMUM
* EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES
* SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS
* SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability of
* this software. By using this software, you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2017 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/
/***********************************************************************************************************************
* File Name   : r_mtr_ctrl_rx23t.c
* Description : The processes of MCU & inverter control layer for RX23T
***********************************************************************************************************************/
/***********************************************************************************************************************
* History : DD.MM.YYYY Version
*         : 05.04.2017 1.00
*         : 07.07.2017 1.01
***********************************************************************************************************************/

/***********************************************************************************************************************
* Includes <System Includes> , "Project Includes"
***********************************************************************************************************************/
#include <stdint.h>
#include <machine.h>
#include "r_mtr_ctrl_rx23t.h"
#include "r_mtr_ctrl_mcu.h"
#include "iodefine.h"
#include "r_mtr_common.h"
#include "r_mtr_parameter.h"

/***********************************************************************************************************************
* Private functions definitions
***********************************************************************************************************************/
static void    mtr_unused_pin_setup(void);     /* Setup unused pins */
static void    mtr_init_clock(void);           /* Initializes clock */
static void    mtr_init_wdt(void);             /* Initializes WDT */
static void    mtr_init_cmt(void);             /* Initializes CMT */
static void    mtr_init_poe3(void);            /* Initializes POE3 */
static void    mtr_init_mtu_3_4(void);         /* Initializes MTU3_3 MTU3_4 */
static void    mtr_init_mtu_0(void);           /* Initializes MTU3_0 */
static void    mtr_init_ad_converter(void);    /* Initializes A/D converter */
static void    mtr_init_irq(void);             /* Initializes external(Hall) interrupt */
static void    mtr_init_register(uint8_t u1_id);
                                               /* Removes unintentional first pulse */

/***********************************************************************************************************************
* Function Name : R_MTR_InitHardware
* Description   : Initializes peripheral function
* Arguments     : None
* Return Value  : None
***********************************************************************************************************************/
void R_MTR_InitHardware(void)
{
    /*****************************/
    /*     Unused pins setup     */
    /*************************** */
    mtr_unused_pin_setup();

    /*=========================*/
    /*     Initialize clock    */
    /*=========================*/
    mtr_init_clock();

    /*========================*/
    /*     Initialize WDT     */
    /*========================*/
    mtr_init_wdt();

    /*========================*/
    /*     Initialize CMT     */
    /*========================*/
    mtr_init_cmt();

    /*=========================*/
    /*     Initialize POE3     */
    /*=========================*/
    mtr_init_poe3();

    /*===================================*/
    /*     Initialize MTU3_3,MTU3_4      */
    /*===================================*/
    mtr_init_mtu_3_4();

    /*============================*/
    /*     initialize MTU3_0      */
    /*============================*/
    mtr_init_mtu_0();

    /*==================================*/
    /*     Initialize A/D converter     */
    /*==================================*/
    mtr_init_ad_converter();

    /*======================================*/
    /*     initialize External Interrupt    */
    /*======================================*/
    mtr_init_irq();
} /* End of function R_MTR_InitHardware */

/***********************************************************************************************************************
* Function Name : R_MTR_InitBoardUi
* Description   : Initializes peripheral function for board user interface
* Arguments     : None
* Return Value  : None
***********************************************************************************************************************/
void R_MTR_InitBoardUi(void)
{
    /*************** SW1,2 ***************/
    PORT9.PDR.BIT.B1 = 0;                               /* Input mode for SW1 */
    PORT9.PDR.BIT.B2 = 0;                               /* Input mode for SW2 */

    /*************** LED1,2,3 ***************/
    PORT0.PODR.BIT.B0 = 1;                              /* LED1 = "H" */
    PORT0.PODR.BIT.B1 = 1;                              /* LED2 = "H" */
    PORT3.PODR.BIT.B1 = 1;                              /* LED3 = "H" */
    PORT0.PDR.BIT.B0 = 1;                               /* Output mode for LED1 */
    PORT0.PDR.BIT.B1 = 1;                               /* Output mode for LED2 */
    PORT3.PDR.BIT.B1 = 1;                               /* Output mode for LED3 */
} /* End of function R_MTR_InitBoardUi */

/***********************************************************************************************************************
* Function Name : R_MTR_ClearWdt
* Description   : Clears watch dog timer
* Arguments     : None
* Return Value  : None
***********************************************************************************************************************/
void R_MTR_ClearWdt(void)
{
    IWDT.IWDTRR = 0x00;                                 /* IWDT Refresh Operation */
    IWDT.IWDTRR = 0xFF;
} /* End of function R_MTR_ClearWdt */

/***********************************************************************************************************************
* Function Name : mtr_init_clock
* Description   : Initializes clock
* Arguments     : None
* Return Value  : None
***********************************************************************************************************************/
static void mtr_init_clock(void)
{
    /* ---- Enable write protection ---- */
    /* PRCR - Protect Register
    b15:b8 PRKEY - PRC Key Code - A5h
                  (The write value should be A5h to permission writing PRCi bit)
    b1     PRC1 - Protect Bit 1 - Write enabled
    b0     PRC0 - Protect Bit 0 - Write enabled */
    SYSTEM.PRCR.WORD = 0xA503;

    /* ----  Set the main clock oscillator drive capability ---- */
    /* MOFCR - Main Clock Oscillator Forced Oscillation Control Register
    b7       Reserved - The write value should be 0.
    b6       MOSEL    - Main Clock Oscillator Switch - Resonator
    b5       MODRV21  - Main Clock Oscillator Drive Capability Switch
                      - 10 MHz to 20 MHz
    b4:b0    Reserved - The write value should be 0. */
    SYSTEM.MOFCR.BYTE = 0x20;

    /* ---- Set wait time until the main clock oscillator stabilizes ---- */
    /* MOSCWTCR - Main Clock Oscillator Wait Control Register
    b7:b5    Reserved - The write value should be 0.
    b4:b0    MSTS     - Main Clock Oscillator Waiting Time
                      - Wait time is 8192 cycles (2.048 ms). */
    SYSTEM.MOSCWTCR.BYTE = 0x04;

    /* ---- Operate the main clock oscillator ---- */
    /* MOSCCR   - Main Clock Oscillator Control Register
    b7:b1    Reserved - The write value should be 0.
    b0       MOSTP    - Main Clock Oscillator Stop - Main clock oscillator is operating. */
    SYSTEM.MOSCCR.BYTE = 0x00;

    /* ---- Wait processing for the clock oscillation stabilization ---- */
    while (1 != SYSTEM.OSCOVFSR.BIT.MOOVF)
    {
        /* Waits until MOOVF bit is set to 1. Main clock can be used after MOOVF bit is set to 1. */
    }

    /* ---- Set the PLL division ratio and multiplication factor ---- */
    /* PLLCR - PLL Control Register
    b15:b14  Reserved - The write value should be 0.
    b13:b8   STC      - Frequency Multiplication Factor Select
                      - Frequency multiplication factor is multiply-by-4.
    b7:b2    Reserved - The write value should be 0.
    b1:b0    PLIDIV   - PLL Input Frequency Division Ratio Select
                      - PLL input division ratio is not divide. */
    SYSTEM.PLLCR.WORD = 0x0700;

    /* ---- Operate the PLL clock oscillator ---- */
    /* PLLCR2 - PLL Control Register 2
    b7:b1    Reserved - The write value should be 0.
    b0       PLLEN    - PLL Stop Control - PLL is operating. */
    SYSTEM.PLLCR2.BYTE = 0x00;

    /* ---- Wait processing for the clock oscillation stabilization ---- */
    while (1 != SYSTEM.OSCOVFSR.BIT.PLOVF)
    {
        /* Waits until PLOVF bit is set to 1. Main clock can be used after PLOVF bit is set to 1. */
    }

    while (0 != SYSTEM.OPCCR.BIT.OPCMTSF)
    {
        /* Check that operation power control mode transition is completed. */
    }

    /* ---- Set the operating power control mode ---- */
    /* OPCCR - Operating Power Control Register
    b7:b5   Reserved - The write value should be 0.
    b4      OPCMTSF  - Operating Power Control Mode Transition Status Flag
    b3      Reserved - The write value should be 0.
    b2:b0   OPCM     - Operating Power Control Mode Select - High-speed operating mode */
    SYSTEM.OPCCR.BYTE = 0x00;

    while (0 != SYSTEM.OPCCR.BIT.OPCMTSF)
    {
        /* Confirm that the operation power control mode transition completed. */
    }

    /* ---- Set the wait cycle of the ROM ---- */
    /* MEMWAIT - Memory Wait Cycle Setting Register
    b7:b1   Reserved - The write value should be 0.
    b0      MEMWAIT  - Memory Wait Cycle Setting - Wait states */
    SYSTEM.MEMWAIT.BYTE = 0x01;

    while (0x01 != SYSTEM.MEMWAIT.BYTE)
    {
        /* Confirm that the written value can be read correctly. */
    }

    /* ---- Set the internal clock division ratio ---- */
    /* SCKCR - System Clock Control Register
    b31:b28 FCK      - FlashIF Clock(FCLK) Select - divide-by-2
    b27:b24 ICK      - System Clock (ICLK) Select - no division
    b23:b18 Reserved - The write value should be 0.
    b17:b16 Reserved - The write value should be 1.
    b12:b15 PCLKA    - Peripheral Module Clock A(PCLKA) Select - no division
    b11:b8  PCLKB    - Peripheral Module Clock B(PCLKB) Select - no division
    b7:b4   Reserved - The write value should be 0.
    b3:b0   PCLKD    - Peripheral Module Clock D(PCLKD) Select - no division */
    SYSTEM.SCKCR.LONG = 0x10030000;
    while (0x10030000 != SYSTEM.SCKCR.LONG)
    {
         /* Confirm that the written value can be read correctly. */
    }

    /* ---- Set the internal clock source ---- */
    /* SCKCR3 - System Clock Control Register 3
    b10:b8  CKSEL    - Clock Source Select - PLL circuit is selected. */
    SYSTEM.SCKCR3.WORD = 0x0400;
    while (0x0400 != SYSTEM.SCKCR3.WORD)
    {
        /* Confirm that the written value can be read correctly. */
    }

    /* ---- Disable write protection ---- */
    /* PRCR - Protect Register
    b15:b8 PRKEY - PRC Key Code - A5h
                  (The write value should be A5h to permission writing PRCi bit)
    b1     PRC1 - Protect Bit 1 - Write disabled
    b0     PRC0 - Protect Bit 0 - Write disabled */
    SYSTEM.PRCR.WORD = 0xA500;
} /* End of function mtr_init_clock */

/***********************************************************************************************************************
* Function Name : mtr_init_wdt
* Description   : Initializes IWDT
* Arguments     : None
* Return Value  : None
***********************************************************************************************************************/
static void mtr_init_wdt(void)
{
    SYSTEM.PRCR.WORD = 0xA503;                          /* PRCR write enabled */
    SYSTEM.ILOCOCR.BIT.ILCSTP = 0;                      /* IWDT-dedicated on-chip oscillator is operating */
    SYSTEM.PRCR.WORD = 0xA500;                          /* PRCR write disabled */
    IWDT.IWDTRR = 0x00;                                 /* IWDT Refresh Operation */
    IWDT.IWDTRR = 0xFF;
} /* End of function mtr_init_wdt */

/***********************************************************************************************************************
* Function Name : mtr_init_cmt
* Description   : Initializes CMT
* Arguments     : None
* Return Value  : None
***********************************************************************************************************************/
static void mtr_init_cmt(void)
{
    SYSTEM.PRCR.WORD = 0xA502;                          /* Enable write protection */
    MSTP_CMT0 = 0;                                      /* The module stop state is canceled */
    SYSTEM.PRCR.WORD = 0xA500;                          /* Disable write protection */

    CMT.CMSTR0.BIT.STR0 = 0;                            /* CMT0 count is stopped */

    CMT0.CMCR.BIT.CKS = 0;                              /* Clock : PCLK/8 = 5[MHz] */

    CMT0.CMCOR = 4999;                                  /* Set interval period : 1 [ms] */
                                                        /* 4999 = (5[MHz] / 1[ms]) - 1 */

    ICU.IPR[IPR_CMT0_CMI0].BIT.IPR  = 11;               /* Interrupt priority level */
    ICU.IR[IR_CMT0_CMI0].BIT.IR     = 0;                /* Clear interrupt request */
    ICU.IER[IER_CMT0_CMI0].BIT.IEN4 = 1;                /* Interrupt request enable */
    CMT0.CMCR.BIT.CMIE              = 1;                /* Compare match interrupt (CMI0) enabled */

    CMT.CMSTR0.BIT.STR0 = 1;                            /* CMT0 count is started */
} /* End of function mtr_init_cmt */

/***********************************************************************************************************************
* Function Name : mtr_init_poe3
* Description   : Initializes POE3 (Over-current protection)
* Arguments     : None
* Return Value  : None
***********************************************************************************************************************/
static void mtr_init_poe3(void)
{
    PORT7.PMR.BIT.B0 = 0;                            /* Use pin as general I/O port */

    MPC.PWPR.BIT.B0WI = 0;                           /* Writing to the PFSWE bit is enabled */
    MPC.PWPR.BIT.PFSWE = 1;                          /* Writing to the PFS register is enabled */

    MPC.P70PFS.BIT.PSEL = 7;                         /* Set peripheral for POE0# */

    MPC.PWPR.BIT.PFSWE = 0;                          /* Writing to the PFS register is disabled */
    MPC.PWPR.BIT.B0WI = 1;                           /* Writing to the PFSWE bit is disabled */

    PORT7.PMR.BIT.B0 = 1;                            /* Use pin as I/O port for peripheral functions */

    POE.ICSR1.BIT.POE0M = 0;                         /* Accepts a request on the falling edge of POE0# input */
    POE.ICSR1.BIT.PIE1 = 1;                          /* OEI1 interrupt requests by the input level detection enabled */

    POE.ALR1.WORD = 0x00BF;                          /* Set MTU output pins for initial output"L",active level"H" */

    ICU.IPR[IPR_POE_OEI1].BIT.IPR = 15;              /* Interrupt priority level */
    ICU.IR[IR_POE_OEI1].BIT.IR = 0;                  /* Clear interrupt request */
    ICU.IER[IER_POE_OEI1].BIT.IEN0 = 1;              /* Interrupt request enable */
    POE.OCSR1.BIT.OCE1 = 1;                          /* Interrupt (OEI1) enabled */
    POE.OCSR1.BIT.OIE1 = 1;                          /* Places the pins in high-impedance */

    POE.POECR2.BIT.MTU3BDZE = 1;                     /* MTU Port 1 High-Impedance enable */
    POE.POECR2.BIT.MTU4ACZE = 1;                     /* MTU Port 2 High-Impedance enable */
    POE.POECR2.BIT.MTU4BDZE = 1;                     /* MTU Port 3 High-Impedance enable */
} /* End of function mtr_init_poe3 */

/***********************************************************************************************************************
* Function Name : mtr_init_mtu_3_4
* Description   : Initializes MTU3 timer for PWM output
* Arguments     : None
* Return Value  : None
***********************************************************************************************************************/
static void mtr_init_mtu_3_4(void)
{
    /***** Module stop state *****/
    SYSTEM.PRCR.WORD = 0xA502;                       /* Enable write protection */
    MSTP_MTU = 0;                                    /* The module stop state is canceled */
    SYSTEM.PRCR.WORD = 0xA500;                       /* Disable write protection */

    /***** Stop count operation *****/
    MTU.TSTRA.BYTE = 0x00;                           /* Stop MTU3_3 and MTU3_4 */

    /***** Count clock selection *****/
    MTU3.TCR.BIT.TPSC   = 0;                         /* MTU3_3 clock : PCLKA */
    MTU3.TCR.BIT.CKEG   = 0;                         /* MTU3_3 count at rising edge */
    MTU3.TCR2.BIT.TPSC2 = 0;
    MTU4.TCR.BIT.TPSC   = 0;                         /* MTU3_4 clock : PCLKA */
    MTU4.TCR.BIT.CKEG   = 0;                         /* MTU3_4 count at rising edge */
    MTU4.TCR2.BIT.TPSC2 = 0;

    /***** TCNT setting *****/
    MTU3.TCNT = MTR_DEADTIME_SET;                    /* Set dead-time */
    MTU4.TCNT = 0;                                   /* Set "0" */

    /***** TGR setting *****/
    MTU3.TGRB = (uint16_t)(MTR_CARRIER_SET_BASE + (MTR_DEADTIME_SET * 0.5f));
    MTU4.TGRA = (uint16_t)(MTR_CARRIER_SET_BASE + (MTR_DEADTIME_SET * 0.5f));
    MTU4.TGRB = (uint16_t)(MTR_CARRIER_SET_BASE + (MTR_DEADTIME_SET * 0.5f));
                                                     /* Set initial duty */
    MTU3.TGRD = (uint16_t)(MTR_CARRIER_SET_BASE + (MTR_DEADTIME_SET * 0.5f));
    MTU4.TGRC = (uint16_t)(MTR_CARRIER_SET_BASE + (MTR_DEADTIME_SET * 0.5f));
    MTU4.TGRD = (uint16_t)(MTR_CARRIER_SET_BASE + (MTR_DEADTIME_SET * 0.5f));
                                                     /* Set initial duty */

    /***** Dead-time and carrier period setting *****/
    MTU.TDDRA = MTR_DEADTIME_SET;                    /* Set dead-time */

    MTU.TCDRA = (uint16_t)(MTR_CARRIER_SET - MTR_DEADTIME_SET);
    MTU.TCBRA = (uint16_t)(MTR_CARRIER_SET - MTR_DEADTIME_SET);
                                                     /* Set carrier period */

    MTU3.TGRA = MTR_CARRIER_SET;                     /* Set "carrier period + dead-time" */
    MTU3.TGRC = MTR_CARRIER_SET;

    /***** Enable PWM output, set PWM output level *****/
    MTU.TOCR1A.BIT.PSYE = 0;                         /* Toggle output is disabled */
    MTU.TOCR1A.BIT.OLSP = 1;                         /* Positive phase : initial output"L",active level"H" */
    MTU.TOCR1A.BIT.OLSN = 1;                         /* Negative phase : initial output"L",active level"H" */

    /***** Complementary PWM mode setting *****/
    MTU3.TMDR1.BIT.MD = 14;                          /* MD[3:0]=0b1110 : complementary PWM mode (transfer at trough) */
    MTU3.TMDR1.BIT.BFA = 1;                          /* TGRA and TGRC used together for buffer operation */
    MTU3.TMDR1.BIT.BFB = 1;                          /* TGRB and TGRD used together for buffer operation */

    /***** Waveform output setting *****/
    MTU.TOERA.BYTE = 0xC0;                           /* PWM output disable  */
    MTU3.TIORH.BYTE = 0x00;                          /* MTIOC3B output prohibited */
    MTU3.TIORL.BYTE = 0x00;                          /* MTIOC3D output prohibited */
    MTU4.TIORH.BYTE = 0x00;                          /* MTIOC4B output prohibited */
    MTU4.TIORL.BYTE = 0x00;                          /* MTIOC4D output prohibited */

    /***** MPC setting *****/
    PORT7.PMR.BIT.B1 = 0;                            /* Use pin as general I/O port */
    PORT7.PMR.BIT.B2 = 0;                            /* Use pin as general I/O port */
    PORT7.PMR.BIT.B3 = 0;                            /* Use pin as general I/O port */
    PORT7.PMR.BIT.B4 = 0;                            /* Use pin as general I/O port */
    PORT7.PMR.BIT.B5 = 0;                            /* Use pin as general I/O port */
    PORT7.PMR.BIT.B6 = 0;                            /* Use pin as general I/O port */

    PORT7.PODR.BIT.B1 = 0;                           /* Up = "L" */
    PORT7.PODR.BIT.B2 = 0;                           /* Vp = "L" */
    PORT7.PODR.BIT.B3 = 0;                           /* Wp = "L" */
    PORT7.PODR.BIT.B4 = 0;                           /* Un = "L" */
    PORT7.PODR.BIT.B5 = 0;                           /* Vn = "L" */
    PORT7.PODR.BIT.B6 = 0;                           /* Wn = "L" */
    PORT7.PDR.BIT.B1 = 1;                            /* Output mode for Up */
    PORT7.PDR.BIT.B2 = 1;                            /* Output mode for Vp */
    PORT7.PDR.BIT.B3 = 1;                            /* Output mode for Wp */
    PORT7.PDR.BIT.B4 = 1;                            /* Output mode for Un */
    PORT7.PDR.BIT.B5 = 1;                            /* Output mode for Vn */
    PORT7.PDR.BIT.B6 = 1;                            /* Output mode for Wn */

    MPC.PWPR.BIT.B0WI = 0;                           /* Writing to the PFSWE bit is enabled */
    MPC.PWPR.BIT.PFSWE = 1;                          /* Writing to the PFS register is enabled */

    MPC.P71PFS.BIT.PSEL = 1;                         /* Set peripheral for MTIOC3B */
    MPC.P72PFS.BIT.PSEL = 1;                         /* Set peripheral for MTIOC4A */
    MPC.P73PFS.BIT.PSEL = 1;                         /* Set peripheral for MTIOC4B */
    MPC.P74PFS.BIT.PSEL = 1;                         /* Set peripheral for MTIOC3D */
    MPC.P75PFS.BIT.PSEL = 1;                         /* Set peripheral for MTIOC4C */
    MPC.P76PFS.BIT.PSEL = 1;                         /* Set peripheral for MTIOC4D */

    MPC.PWPR.BIT.PFSWE = 0;                          /* Writing to the PFS register is disabled */
    MPC.PWPR.BIT.B0WI = 1;                           /* Writing to the PFSWE bit is disabled */

    PORT7.PMR.BIT.B1 = 1;                            /* Use pin as I/O port for peripheral functions */
    PORT7.PMR.BIT.B2 = 1;                            /* Use pin as I/O port for peripheral functions */
    PORT7.PMR.BIT.B3 = 1;                            /* Use pin as I/O port for peripheral functions */
    PORT7.PMR.BIT.B4 = 1;                            /* Use pin as I/O port for peripheral functions */
    PORT7.PMR.BIT.B5 = 1;                            /* Use pin as I/O port for peripheral functions */
    PORT7.PMR.BIT.B6 = 1;                            /* Use pin as I/O port for peripheral functions */

    /***** Interrupt setting *****/
    MTU4.TIER.BIT.TCIEV = 1;                /* Enable underflow interrupt when operation is in complementary PWM mode */
    MTU4.TIER.BIT.TTGE2 = 1;                /* Enable TCNT underflow A/D convert */
    MTU4.TIER.BIT.TTGE = 0;                 /* Enable A/D convert request */

    ICU.IPR[IPR_MTU4_TCIV4].BIT.IPR = 12;            /* Interrupt priority level */
    ICU.IR[IR_MTU4_TCIV4].BIT.IR = 0;                /* Clear interrupt request */
    ICU.IER[IER_MTU4_TCIV4].BIT.IEN2 = 1;            /* Interrupt request is enable */

    MTU.TITMRA.BIT.TITM = 0;                         /* Set interrupt skipping function */
    MTU.TITCR1A.BIT.T4VCOR = MTR_INT_DECIMATION;     /* Interrupt skip number of time */
    MTU.TITCR1A.BIT.T4VEN = 1;                       /* Enable TCIV4 interrupt skip */

    /***** Start count operation *****/
    MTU.TSTRA.BYTE = 0xC0;                           /* Start MTU3_3 and MTU3_4 */
} /* End of function mtr_init_mtu_3_4 */

/***********************************************************************************************************************
* Function Name : mtr_init_mtu_0
* Description   : Initialize MTU3 Encoder counter
* Arguments     : None
* Return Value  : None
***********************************************************************************************************************/
static void mtr_init_mtu_0(void)
{
    /***** module stop state *****/
    SYSTEM.PRCR.WORD = 0xA502;                          /* enable write protection */
    MSTP_MTU = 0;                                       /* the module stop state is canceled */
    SYSTEM.PRCR.WORD = 0xA500;                          /* disable write protection */

    /***** encoder timer setting *****/
    PORT3.PMR.BIT.B2 = 0;                               /* use pin as general I/O port */
    PORT3.PMR.BIT.B3 = 0;                               /* use pin as general I/O port */

    PORT3.PDR.BIT.B2 = 0;                               /* input mode */
    PORT3.PDR.BIT.B3 = 0;                               /* input mode */

    MPC.PWPR.BIT.B0WI = 0;                              /* writing to the PFSWE bit is enabled */
    MPC.PWPR.BIT.PFSWE = 1;                             /* writing to the PFS register is enabled */

    MPC.P32PFS.BIT.PSEL = 2;                            /* set peripheral for MTCLKB */
    MPC.P33PFS.BIT.PSEL = 2;                            /* set peripheral for MTCLKA */

    MPC.PWPR.BIT.PFSWE = 0;                             /* writing to the PFS register is disabled */
    MPC.PWPR.BIT.B0WI = 1;                              /* writing to the PFSWE bit is disabled */

    PORT3.PMR.BIT.B2 = 1;                               /* use pin as I/O port for peripheral functions */
    PORT3.PMR.BIT.B3 = 1;                               /* use pin as I/O port for peripheral functions */

    MTU1.TMDR1.BIT.MD  = 4;                             /* MD[3:0]=0b0100:phase counting mode 1 */
    MTU1.TCR.BIT.CCLR  = 0;                             /* TCNT clear prohibition */
    MTU1.TGRA          = 0;                             /* clear counter */
    MTU.TSTRA.BIT.CST1 = 1;                             /* start MTU3_1 */

    MTU0.TCR.BIT.TPSC = 0;                              /* MTU3_0 clock : PCLKA */
    MTU0.TCR.BIT.CKEG = 0;                              /* MTU3_0 count at rising edge */
    MTU0.TCR.BIT.CCLR = 0;                              /* MTU3_0 clearing disable */
    MTU0.TMDR1.BIT.MD = 0;                              /* MTU3_0 normal mode */
    MTU0.TMDR1.BIT.BFA = 0;                             /* TGRA and TGRC used together for non-buffer operation */
    MTU0.TMDR1.BIT.BFB = 0;                             /* TGRB and TGRD used together for non-buffer operation */
    MTU0.TMDR1.BIT.BFE = 0;                             /* TGRE and TGRF used together for non-buffer operation */
    MTU0.TIORH.BIT.IOB = 12;                            /* MTU1.TCNT count signal input capture */

    /* MTU0_TGIE interrupt enable */
    MTU0.TIER.BIT.TGIEB              = 1;
    ICU.IPR[IPR_MTU0_TGIB0].BIT.IPR  = 13;              /* interrupt priority level */
    ICU.IR[IR_MTU0_TGIB0].BIT.IR     = 0;
    ICU.IER[IER_MTU0_TGIB0].BIT.IEN3 = 1;

    MTU0.TCNT = 0;
    MTU.TSTRA.BIT.CST0 = 0;                             /* stop MTU3_0 */
} /* End of function mtr_init_mtu_0 */

/***********************************************************************************************************************
* Function Name : mtr_init_ad_converter
* Description   : Initializes A/D converter
* Arguments     : None
* Return Value  : None
***********************************************************************************************************************/
static void mtr_init_ad_converter(void)
{
    SYSTEM.PRCR.WORD = 0xA502;                          /* Enable write protection */
    MSTP_S12AD = 0;                                     /* The module stop state is canceled */
    SYSTEM.PRCR.WORD = 0xA500;                          /* Disable write protection */

    S12AD.ADCSR.BIT.EXTRG = 0;                          /* A/D conversion is started by synchronous trigger */
    S12AD.ADCSR.BIT.TRGE = 1;                           /* Disables A/D conversion trigger */
    S12AD.ADCSR.BIT.ADIE = 0;                           /* Disables S12ADI0 interrupt generation upon scan completion */
    S12AD.ADCSR.BIT.ADCS = 0;                           /* Single scan mode */

    MPC.PWPR.BIT.B0WI = 0;                              /* Writing to the PFSWE bit is enabled */
    MPC.PWPR.BIT.PFSWE = 1;                             /* Writing to the PFS register is enabled */

    MPC.P40PFS.BIT.ASEL = 1;                            /* Pin Function Select : AN000 (IU) */
    MPC.P42PFS.BIT.ASEL = 1;                            /* Pin Function Select : AN002 (IW) */
    MPC.P43PFS.BIT.ASEL = 1;                            /* Pin Function Select : AN003 (VPN) */
    MPC.P10PFS.BIT.ASEL = 1;                            /* Pin Function Select : AN017 (VR1) */

    MPC.PWPR.BIT.PFSWE = 0;                             /* Writing to the PFS register is disabled */
    MPC.PWPR.BIT.B0WI = 1;                              /* Writing to the PFSWE bit is disabled */

    S12AD.ADANSA0.WORD = 0x000D;                        /* A/D convert channel select AN000,AN002,AN003 */
    S12AD.ADANSA1.WORD = 0x0002;                        /* A/D convert channel select AN017 */

    S12AD.ADSHCR.BIT.SHANS = 5;                         /* Enable sampling & hold AN000,AN002 */
    S12AD.ADCER.BIT.ADRFMT = 0;                         /* A/D data format "flush-right" */

    S12AD.ADSTRGR.BIT.TRSA = 5;                         /* A/D convert trigger MTU4.TCNT underflow */
} /* End of function mtr_init_ad_converter */

/***********************************************************************************************************************
* Function Name : mtr_init_register
* Description   : Avoid unintentional first pulse
* Arguments     : u1_id - Motor ID
* Return Value  : None
***********************************************************************************************************************/
static void mtr_init_register(uint8_t u1_id)
{
    uint16_t u2_temp;

    switch (u1_id)
    {
        case MTR_ID_A:
            u2_temp = (uint16_t)(MTR_CARRIER_SET_BASE + (MTR_DEADTIME_SET * 0.5f));

            MTU3.TGRD = u2_temp;
            MTU4.TGRC = u2_temp;
            MTU4.TGRD = u2_temp;                        /* Set duty */
        break;

        default:
            /* Do Nothing */
        break;
    }
    mtr_inv_set_uvw(0.5f, 0.5f, 0.5f, u1_id);
} /* End of function mtr_init_register */

/***********************************************************************************************************************
* Function Name : mtr_init_irq
* Description   : Initialize of External(Hall) Interrupt
* Arguments     : None
* Return Value  : None
***********************************************************************************************************************/
static void mtr_init_irq(void)
{
    /*************** PORT Setting ******************/
    MPC.PWPR.BIT.B0WI = 0;                              /* Writing to the PFSWE bit is enabled */
    MPC.PWPR.BIT.PFSWE = 1;                             /* Writing to the PFS register is enabled */

    MPC.P93PFS.BIT.ISEL = 1;                            /* PORT93 is used as IRQ0. */
    MPC.P94PFS.BIT.ISEL = 1;                            /* PORT94 is used as IRQ1. */
    MPC.PA2PFS.BIT.ISEL = 1;                            /* PORTA2 is used as IRQ4. */

    PORT9.PMR.BIT.B3 = 1;
    PORT9.PDR.BIT.B3 = 0;                               /* input mode for HU */

    PORT9.PMR.BIT.B4 = 1;
    PORT9.PDR.BIT.B4 = 0;                               /* input mode for HV */

    PORTA.PMR.BIT.B2 = 1;
    PORTA.PDR.BIT.B2 = 0;                               /* input mode for HW */

    MPC.PWPR.BIT.PFSWE = 0;                             /* Writing to the PFS register is disabled */
    MPC.PWPR.BIT.B0WI = 1;                              /* Writing to the PFSWE bit is disabled */

    /*************** IRQ0 (HU) ***************/
    ICU.IER[IER_ICU_IRQ0].BIT.IEN0 = 0;                 /* IRQ0 interrupt disable */
    ICU.IRQFLTE0.BIT.FLTEN0 = 0;                        /* digital filter disable */
    ICU.IRQFLTC0.BIT.FCLKSEL0 = 0x00;                   /* digital filter timing:PCLK */
    ICU.IRQCR[0].BIT.IRQMD = 0x03;                      /* 0b11:use both edge */
    ICU.IR[IR_ICU_IRQ0].BIT.IR = 0;                     /* IRQ0 interrupt requirement enable */
    ICU.IRQFLTE0.BIT.FLTEN0 = 1;                        /* digital filter enable */
    ICU.IPR[IPR_ICU_IRQ0].BIT.IPR = 14;                 /* interrupt priority level */

    /*************** IRQ1 (HV) ***************/
    ICU.IER[IER_ICU_IRQ1].BIT.IEN1 = 0;                 /* IRQ1 interrupt disable */
    ICU.IRQFLTE0.BIT.FLTEN1 = 0;                        /* digital filter disable */
    ICU.IRQFLTC0.BIT.FCLKSEL1 = 0x00;                   /* digital filter timing:PCLK */
    ICU.IRQCR[1].BIT.IRQMD = 0x03;                      /* 0b11:use both edge */
    ICU.IR[IR_ICU_IRQ1].BIT.IR = 0;                     /* IRQ1 interrupt requirement enable */
    ICU.IRQFLTE0.BIT.FLTEN1 = 1;                        /* digital filter enable */
    ICU.IPR[IPR_ICU_IRQ1].BIT.IPR = 14;                 /* interrupt priority level */

    /*************** IRQ4 (HW) ***************/
    ICU.IER[IER_ICU_IRQ4].BIT.IEN4 = 0;                 /* IRQ4 interrupt disable */
    ICU.IRQFLTE0.BIT.FLTEN4 = 0;                        /* digital filter disable */
    ICU.IRQFLTC0.BIT.FCLKSEL4 = 0x00;                   /* digital filter timing:PCLK */
    ICU.IRQCR[4].BIT.IRQMD = 0x03;                      /* 0b11:use both edge */
    ICU.IR[IR_ICU_IRQ4].BIT.IR = 0;                     /* IRQ4 interrupt requirement enable */
    ICU.IRQFLTE0.BIT.FLTEN4 = 1;                        /* digital filter enable */
    ICU.IPR[IPR_ICU_IRQ4].BIT.IPR = 14;                 /* interrupt priority level */
} /* End of function mtr_init_irq */

/***********************************************************************************************************************
* Function Name : mtr_encd_interrupt_enable
* Description   : Enables encoder interrupt
* Arguments     : u1_id - Motor ID
* Return Value  : None
***********************************************************************************************************************/
void mtr_encd_interrupt_enable(uint8_t u1_id)
{
    switch (u1_id)
    {
        case MTR_ID_A:
            /* Encoder interrupt enable */
            if(0 == ICU.IER[IER_MTU0_TGIB0].BIT.IEN3)
            {
                ICU.IER[IER_MTU0_TGIB0].BIT.IEN3 = 1;   /* Encoder interrupt enable */
            }
        break;

        default:
            /* Do Nothing */
        break;
    }
} /* End of function mtr_encd_interrupt_enable */

/***********************************************************************************************************************
* Function Name : mtr_encd_interrupt_disable
* Description   : Disables encoder interrupt
* Arguments     : u1_id - Motor ID
* Return Value  : None
***********************************************************************************************************************/
void mtr_encd_interrupt_disable(uint8_t u1_id)
{
    switch (u1_id)
    {
        case MTR_ID_A:
            /* Encoder interrupt disable */
            if(1 == ICU.IER[IER_MTU0_TGIB0].BIT.IEN3)
            {
                ICU.IER[IER_MTU0_TGIB0].BIT.IEN3 = 0;   /* Encoder interrupt disable */
            }
        break;

        default:
            /* Do Nothing */
        break;
    }
} /* End of function mtr_encd_interrupt_disable */

/***********************************************************************************************************************
* Function Name : mtr_irq_interrupt_enable
* Description   : Enables external(Hall) interrupt
* Arguments     : u1_id - Motor ID
* Return Value  : None
***********************************************************************************************************************/
void mtr_irq_interrupt_enable(uint8_t u1_id)
{
    switch (u1_id)
    {
        case MTR_ID_A:
            /* Hall interrupt enable */
            if((0 == ICU.IER[IER_ICU_IRQ0].BIT.IEN0) ||
               (0 == ICU.IER[IER_ICU_IRQ1].BIT.IEN1) ||
               (0 == ICU.IER[IER_ICU_IRQ4].BIT.IEN4))
                {
                    ICU.IER[IER_ICU_IRQ0].BIT.IEN0 = 1;         /* IRQ0 interrupt enable */
                    ICU.IER[IER_ICU_IRQ1].BIT.IEN1 = 1;         /* IRQ1 interrupt enable */
                    ICU.IER[IER_ICU_IRQ4].BIT.IEN4 = 1;         /* IRQ4 interrupt enable */
            }
        break;

        default:
            /* Do Nothing */
        break;
    }
} /* End of function mtr_irq_interrupt_enable */

/***********************************************************************************************************************
* Function Name : mtr_irq_interrupt_disable
* Description   : Disables external(Hall) interrupt
* Arguments     : u1_id - Motor ID
* Return Value  : None
***********************************************************************************************************************/
void mtr_irq_interrupt_disable(uint8_t u1_id)
{
    switch (u1_id)
    {
        case MTR_ID_A:
            /* Hall interrupt disable */
            if((1 == ICU.IER[IER_ICU_IRQ0].BIT.IEN0) ||
               (1 == ICU.IER[IER_ICU_IRQ1].BIT.IEN1) ||
               (1 == ICU.IER[IER_ICU_IRQ4].BIT.IEN4))
                {
                    ICU.IER[IER_ICU_IRQ0].BIT.IEN0 = 0;             /* IRQ0 interrupt disable */
                    ICU.IER[IER_ICU_IRQ1].BIT.IEN1 = 0;             /* IRQ1 interrupt disable */
                    ICU.IER[IER_ICU_IRQ4].BIT.IEN4 = 0;             /* IRQ4 interrupt disable */
            }
        break;

        default:
            /* Do Nothing */
        break;
    }
} /* End of function mtr_irq_interrupt_disable */

/***********************************************************************************************************************
* Function Name : mtr_unused_pin_setup
* Description   : Setups unused pins
*                 If an unused pin is floating, it's direction should be set to "output" (PORTn.PDR = 1),
* Arguments     : None
* Return Value  : None
***********************************************************************************************************************/
static void mtr_unused_pin_setup(void)
{
    /* Set TX pins for ICS connectors to output mode and Low level */
    /****** CN2(SCI1) ******/
    PORTD.PDR.BIT.B3 = 1;                               /* Pin No.2 */
    /****** CN3(SCI5-1) ******/
    PORTB.PDR.BIT.B6 = 1;                               /* Pin No.2 */

    /* Set unused pins on CN5 to output mode and Low level */
    PORT2.PDR.BIT.B2 = 1;                               /* Pin No.5 */
    PORT2.PDR.BIT.B3 = 1;                               /* Pin No.6 */
    PORT2.PDR.BIT.B4 = 1;                               /* Pin No.7 */
    PORT3.PDR.BIT.B0 = 1;                               /* Pin No.8 */
    PORTA.PDR.BIT.B4 = 1;                               /* Pin No.9 */
    PORT0.PDR.BIT.B2 = 1;                               /* Pin No.10 */
    PORTA.PDR.BIT.B3 = 1;                               /* Pin No.14 */
    PORTB.PDR.BIT.B0 = 1;                               /* Pin No.15 */
    PORTB.PDR.BIT.B3 = 1;                               /* Pin No.16 */
    PORTB.PDR.BIT.B7 = 1;                               /* Pin No.17 */
    PORTD.PDR.BIT.B4 = 1;                               /* Pin No.18 */
    PORTD.PDR.BIT.B6 = 1;                               /* Pin No.19 */
    PORTD.PDR.BIT.B7 = 1;                               /* Pin No.20 */
} /* End of function mtr_unused_pin_setup */

/***********************************************************************************************************************
* Function Name : mtr_ctrl_start
* Description   : Starts PWM output
* Arguments     : u1_id - Motor ID
* Return Value  : None
***********************************************************************************************************************/
void mtr_ctrl_start(uint8_t u1_id)
{
    mtr_init_register(u1_id);
    switch (u1_id)
    {
        case MTR_ID_A:
            MTR_PORT_UP = 0;                            /* Up = "L" */
            MTR_PORT_UN = 0;                            /* Un = "L" */
            MTR_PORT_VP = 0;                            /* Vp = "L" */
            MTR_PORT_VN = 0;                            /* Vn = "L" */
            MTR_PORT_WP = 0;                            /* Wp = "L" */
            MTR_PORT_WN = 0;                            /* Wn = "L" */

            PORT7.PMR.BIT.B1 = 1;                       /* I/O port for peripheral functions */
            PORT7.PMR.BIT.B2 = 1;                       /* I/O port for peripheral functions */
            PORT7.PMR.BIT.B3 = 1;                       /* I/O port for peripheral functions */
            PORT7.PMR.BIT.B4 = 1;                       /* I/O port for peripheral functions */
            PORT7.PMR.BIT.B5 = 1;                       /* I/O port for peripheral functions */
            PORT7.PMR.BIT.B6 = 1;                       /* I/O port for peripheral functions */
            MTU.TOERA.BYTE = 0xFF;                      /* PWM output enable */
        break;

        default:
            /* Do Nothing */
        break;
    }
} /* End of function mtr_ctrl_start */

/***********************************************************************************************************************
* Function Name : mtr_ctrl_stop
* Description   : Stops PWM output disable for motor control
* Arguments     : u1_id - Motor ID
* Return Value  : None
***********************************************************************************************************************/
void mtr_ctrl_stop(uint8_t u1_id)
{
    switch (u1_id)
    {
        case MTR_ID_A:
            MTU.TOERA.BYTE = 0xC0;                      /* PWM output disable */
            PORT7.PMR.BIT.B1 = 0;                       /* General I/O port */
            PORT7.PMR.BIT.B2 = 0;                       /* General I/O port */
            PORT7.PMR.BIT.B3 = 0;                       /* General I/O port */
            PORT7.PMR.BIT.B4 = 0;                       /* General I/O port */
            PORT7.PMR.BIT.B5 = 0;                       /* General I/O port */
            PORT7.PMR.BIT.B6 = 0;                       /* General I/O port */
            MTR_PORT_UP = 0;                            /* Up = "L" */
            MTR_PORT_UN = 0;                            /* Un = "L" */
            MTR_PORT_VP = 0;                            /* Vp = "L" */
            MTR_PORT_VN = 0;                            /* Vn = "L" */
            MTR_PORT_WP = 0;                            /* Wp = "L" */
            MTR_PORT_WN = 0;                            /* Wn = "L" */
        break;

        default:
            /* Do Nothing */
        break;
    }
    mtr_init_register(u1_id);
} /* End of function mtr_ctrl_stop */

/***********************************************************************************************************************
* Function Name : mtr_get_vr1_adc
* Description   : Gets the A/D conversion result for VR1
* Arguments     : None
* Return Value  : A/D converted value
***********************************************************************************************************************/
uint16_t mtr_get_vr1_adc(void)
{
    uint16_t u2_temp;

    while (1 == S12AD.ADCSR.BIT.ADST)
    {
        /* Wait until AD conversion finish */
    }
    u2_temp = S12AD.ADDR17;

    return (u2_temp);
} /* End of function mtr_get_vr1_adc */

/***********************************************************************************************************************
* Function Name : mtr_inv_set_uvw
* Description   : PWM duty setting
* Arguments     : f4_duty_u - The duty cycle of Phase-U (0.0 - 1.0)
*                 f4_duty_v - The duty cycle of Phase-V (0.0 - 1.0)
*                 f4_duty_w - The duty cycle of Phase-W (0.0 - 1.0)
*                 u1_id     - The ID of motor connector
* Return Value  : None
***********************************************************************************************************************/
void mtr_inv_set_uvw(float f4_duty_u, float f4_duty_v, float f4_duty_w, uint8_t u1_id)
{
    uint16_t u2_count_u;
    uint16_t u2_count_v;
    uint16_t u2_count_w;

    u2_count_u = (uint16_t)((MTR_CARRIER_SET_BASE * (1.0f - f4_duty_u)) + (MTR_DEADTIME_SET * 0.5f));
    u2_count_v = (uint16_t)((MTR_CARRIER_SET_BASE * (1.0f - f4_duty_v)) + (MTR_DEADTIME_SET * 0.5f));
    u2_count_w = (uint16_t)((MTR_CARRIER_SET_BASE * (1.0f - f4_duty_w)) + (MTR_DEADTIME_SET * 0.5f));

    switch (u1_id)
    {
        case MTR_ID_A:
        {
            /* Write general registers of timer */
            MTU3.TGRD = (uint16_t)u2_count_u;
            MTU4.TGRC = (uint16_t)u2_count_v;
            MTU4.TGRD = (uint16_t)u2_count_w;
        break;
        }
        default:
        {
            /* Do Nothing */
        break;
        }
    }
} /* End of function mtr_inv_set_uvw */

/***********************************************************************************************************************
* Function Name : mtr_get_current_adc
* Description   : Gets the A/D conversion result for Iu,Iw
* Arguments     : f4_iu_ad - Where to store the A/D conversion result of U-phase
*                 f4_iw_ad - Where to store the A/D conversion result of W-phase
*                 u1_id - Motor ID
* Return Value  : None
***********************************************************************************************************************/
void mtr_get_current_adc(float *f4_iu_ad, float *f4_iw_ad, uint8_t u1_id)
{
    switch (u1_id)
    {
        case MTR_ID_A:
            while (1 == S12AD.ADCSR.BIT.ADST)
            {
                /* Wait until AD conversion finish */
            }
            *f4_iu_ad = (float)S12AD.ADDR0;
            *f4_iw_ad = (float)S12AD.ADDR2;
        break;

        default:
            /* Do Nothing */
        break;
    }
} /* End of function mtr_get_current_adc */

/***********************************************************************************************************************
* Function Name : mtr_get_vdc_adc
* Description   : A/D conversion for Vdc
* Arguments     : u1_id - Motor ID
* Return Value  : A/D converted value
***********************************************************************************************************************/
float mtr_get_vdc_adc(uint8_t u1_id)
{
    float f4_temp;

    switch (u1_id)
    {
        case MTR_ID_A:
            while (1 == S12AD.ADCSR.BIT.ADST)
            {
                /* Wait until AD conversion finish */
            }
            f4_temp = (float)S12AD.ADDR3;
        break;

        default:
            /* Do Nothing */
        break;
    }

    return (f4_temp);
} /* End of function mtr_get_vdc_adc */

/***********************************************************************************************************************
* Function Name : mtr_speed_calc_timer_start
* Description   : Starts speed calculation timer
* Arguments     : u1_id - Motor ID
* Return Value  : None
***********************************************************************************************************************/
void mtr_speed_calc_timer_start(uint8_t u1_id)
{
    switch (u1_id)
    {
        case MTR_ID_A:
            MTU0.TGRB = 0;                               /* Clear general register */
            MTU0.TGRD = 0;                               /* Clear buffer register */
            MTU0.TCNT = 0;                               /* Clear timer counter */
            MTU.TSTRA.BIT.CST0 = 1;                      /* Start speed calculation timer (MTU3_0) */
        break;

        default:
            /* Do Nothing */
        break;
    }
} /* End of function mtr_speed_calc_timer_start */

/***********************************************************************************************************************
* Function Name : mtr_clear_oc_flag
* Description   : Clears forced cutoff flag (detect over current)
* Arguments     : None
* Return Value  : None
***********************************************************************************************************************/
void mtr_clear_oc_flag(void)
{
    ICU.IR[IR_POE_OEI1].BIT.IR = 0;                     /* Release from high-impedance state */
    POE.ICSR1.BIT.POE0F = 0;                            /* POE0# high-impedance request clear */
    POE.OCSR1.BIT.OSF1 = 0;                             /* Output short circuit clear */
} /* End of function mtr_clear_oc_flag */
