/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products. No
* other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED. TO THE MAXIMUM
* EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES
* SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS
* SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability of
* this software. By using this software, you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2017 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/
/***********************************************************************************************************************
* File Name   : r_mtr_ctrl_encoder.h
* Description : Definitions of encoder control processes
***********************************************************************************************************************/
/**********************************************************************************************************************
* History : DD.MM.YYYY Version
*         : 05.04.2017 1.00
*         : 07.07.2017 1.01
***********************************************************************************************************************/

/* guard against multiple inclusion */
#ifndef R_MTR_CTRL_ENCODER_H
#define R_MTR_CTRL_ENCODER_H

/***********************************************************************************************************************
* Includes <System Includes> , "Project Includes"
***********************************************************************************************************************/
#include "r_mtr_parameter.h"

/***********************************************************************************************************************
* Macro definitions
***********************************************************************************************************************/
/* position and speed calculation mode */
#define MTR_PS_CALC_ENCD_INTERRUPT (0)
#define MTR_PS_CALC_CTRL_INTERRUPT (1)
/* encoder angle adjustment reference status */
#define MTR_ENCD_ANGLE_ADJ_90DEG   (0)
#define MTR_ENCD_ANGLE_ADJ_0DEG    (1)
#define MTR_ENCD_ANGLE_ADJ_FIN     (2)
/* encoder configuration */
#define MTR_ENCD_CPR_MECH          (1200)         /* encoder counts per revolution (mechanical) */
#define MTR_ENCD_ANGLE_ADJ_TIME    (128)          /* encoder angle adjustment time [ms] */
#define MTR_ENCD_CNTAVG            (4)            /* encoder pulse width average */
#define MTR_ENCD_ZEROSPEED_PW      (20000000)     /* encoder pulse width[counts] considered as zero speed */
#define MTR_ENCD_ANGLE_DIFF        (MTR_TWOPI / (float)MTR_ENCD_CPR_MECH)
                                                  /* encoder minimum angle [rad]*/
#define MTR_ENCD_CALC_AVG          (MTR_ENCD_CNTAVG * MTR_ENCD_TIMER_FREQ * 1000000.0f)
                                                  /* encoder avg calc parameter */
#define MTR_CTRL_OCCUPANCY_TIME    (0.48f)        /* occupancy time of carrier interrupt */
#define MTR_CTRL_PROS_TIME         (0.00004310f)  /* processing time of carrier interrupt[s] */
#define MTR_ENCD_PROS_TIME         (0.00000305f)  /* processing time of encoder interrupt[s] */
#define MTR_ENCD_NUMB_OF_TIME      (((MTR_CTRL_PERIOD * MTR_CTRL_OCCUPANCY_TIME-MTR_CTRL_PROS_TIME) / MTR_ENCD_PROS_TIME))
                                                  /* the number of times of encoder interrupt for control period */
#define MTR_HS_CHANGE_RPM          ((MTR_ENCD_NUMB_OF_TIME * 60) / (MTR_CTRL_PERIOD * MTR_ENCD_CPR_MECH))
                                                  /* toggle speed for high speed mode [rpm] (mechanical) */
#define MTR_HS_CHANGE_RAD          ((float)(MTR_HS_CHANGE_RPM * MTR_RPM_RAD))
                                                  /* toggle speed for high speed mode [rad/s] (electrical) */
#define MTR_HS_CHANGE_MARGIN_RPM   (150)          /* margin of toggle speed for high speed mode [rpm] (mechanical)*/
#define MTR_HS_CHANGE_MARGIN_RAD   (MTR_HS_CHANGE_MARGIN_RPM*MTR_RPM_RAD)
                                                  /* margin of toggle speed for high speed mode [rad/s] (electrical) */
#define MTR_HS_SPEED_LPF_PARAM     (0.1f)         /* Highspeed mode speed LPF parameter */
#define MTR_PS_CALC_CHANGE_CNT     (8)            /* counts for mode change of position speed calculation */

/***********************************************************************************************************************
* Global structure
***********************************************************************************************************************/
typedef struct
{
    /* encoder */
    uint8_t  u1_encd_angle_adj_status;                /* encoder angle adjustment status */
    uint16_t u2_encd_angle_adj_time;                  /* time of encoder angle adjustment */
    uint16_t u2_encd_angle_adj_cnt;                   /* counter of encoder angle adjustment */
    uint16_t u2_encd_pre_phase_cnt;                   /* previous value of encoder phase counter */
    uint16_t u2_encd_timer_cap_tcnt;                  /* current MTU0.TCNT resister value */
    uint16_t u2_encd_timer_cap_pre_tcnt;              /* previous MTU0.TCNT resister value */
    uint16_t u2_encd_timer_cnt_num;                   /* s4_encd_timer_cnt_buff index */
    uint16_t u2_encd_cpr_mech;                        /* encoder counts per revolution (mechanical) */
    int32_t  s4_encd_timer_cnt_buff[MTR_ENCD_CNTAVG]; /* stored width of captured pulses */
    int32_t  s4_encd_pulse_width;                     /* stored captured pulse width */
    int32_t  s4_encd_pulse_width_buff;                /* stored captured pulse width buffer */
    int32_t  s4_encd_pulse_width_sum;                 /* sum of last 4 pulse width */
    int32_t  s4_encd_angle_cnt;                       /* counter of encoder position */
    float    f4_encd_angle_diff;                      /* angle of encoder 1 pulse width */
    float    f4_encd_cpr_mech_inv;                    /* inverse number of encoder counts per revolution (mechanical) */
    float    f4_encd_speed_pre_rad;                   /* encoder speed of previous edge detection */
    float    f4_encd_speed_rad;                       /* encoder speed of current edge detection [rad/s] (mechanical) */
    float    f4_encd_pos_rad;                         /* encoder rotor position angle [rad] (mechanical) */
} mtr_encoder_t;

typedef struct
{
    /* encoder high speed */
    uint8_t  u1_encd_pos_speed_calc_mode;             /* position and speed calculation mode */
    uint8_t  u1_encd_interrupt_on_flag;               /* flag of encoder interrupt */
    uint8_t  u1_encd_pos_speed_calc_cnt;              /*`change counter */
    uint16_t u2_encd_hs_pre_phase_cnt;                /* previous value of encoder phase counter for high speed mode */
    int32_t  s4_encd_hs_angle_cnt;                    /* counter of encoder position for high speed mode  */
    float    f4_encd_hs_pos_rad;                      /* encoder rotor position angle [rad]
                                                         (mechanical) for high speed mode */
    float    f4_encd_hs_pos_pre_rad;                  /* previous value of encoder rotor position angle [rad]
                                                         (mechanical) for high speed mode */
    float    f4_encd_hs_speed_rad;                    /* encoder speed of current edge detection [rad/s] (mechanical)
                                                         for high speed mode */
    float    f4_encd_hs_speed_pre_rad;                /* previous value of encoder speed of current edge detection 
                                                         [rad/s] (mechanical) for high speed mode */
    float    f4_encd_hs_sw_speed_rad;                 /* toggle speed for high speed mode [rad/s] (mechanical) */
    float    f4_encd_hs_sw_speed_margin_rad;          /* margin of toggle speed for high speed mode [rad/s]
                                                         (mechanical)*/
} mtr_encoder_highspeed_t;

/***********************************************************************************************************************
* Global function definitions
***********************************************************************************************************************/
/***********************************************************************************************************************
* Function Name : mtr_encoder_init
* Description   : Initialize variables when encoder control
* Arguments     : st_ec - encoder structure (pointer)
* Return Value  : None
***********************************************************************************************************************/
void mtr_encoder_init(mtr_encoder_t *st_ec);

/***********************************************************************************************************************
* Function Name : mtr_encoder_reset
* Description   : Reset variables when encoder control start
* Arguments     : st_ec - encoder structure (pointer)
* Return Value  : None
***********************************************************************************************************************/
void mtr_encoder_reset(mtr_encoder_t *st_ec);

/***********************************************************************************************************************
* Function Name : mtr_encoder_highspeed_init
* Description   : Initialize variables when motor control
* Arguments     : st_ehc - encoder highspeed structure (pointer)
* Return Value  : None
***********************************************************************************************************************/
void mtr_encoder_highspeed_init(mtr_encoder_highspeed_t *st_ehc);

/***********************************************************************************************************************
* Function Name : mtr_encoder_highspeed_reset
* Description   : Reset variables when motor control start
* Arguments     : st_ehc - encoder highspeed structure (pointer)
* Return Value  : None
***********************************************************************************************************************/
void mtr_encoder_highspeed_reset(mtr_encoder_highspeed_t *st_ehc);

/***********************************************************************************************************************
* Function Name : mtr_set_param_encd_cpr_mech
* Description   : Set encoder pulse (quad edge evaluation)
* Arguments     : st_ec            - encoder structure (pointer)
*                 u2_encd_cpr_mech - encoder pulse (quad edge evaluation)
* Return Value  : None
***********************************************************************************************************************/
void mtr_set_param_encd_cpr_mech(mtr_encoder_t *st_ec, uint16_t u2_encd_cpr_mech);

/***********************************************************************************************************************
* Function Name : mtr_set_param_encd_hs_sw_speed_rad
* Description   : Set toggle speed for high speed mode
* Arguments     : st_ehc                  - encoder highspeed structure (pointer)
*                 f4_encd_hs_sw_speed_rpm - toggle speed for high speed mode [rpm]
*                 f4_rpm_rad              - rpm to rad
* Return Value  : None
***********************************************************************************************************************/
void mtr_set_param_encd_hs_sw_speed_rad(mtr_encoder_highspeed_t *st_ehc,
                                        uint16_t f4_encd_hs_sw_speed_rpm,
                                        float f4_rpm_rad);

/***********************************************************************************************************************
* Function Name : mtr_set_param_encd_hs_sw_speed_margin_rad
* Description   : Set margin of toggle speed for high speed mode [rad/s]
* Arguments     : st_ehc                         - encoder highspeed structure (pointer)
*                 f4_encd_hs_sw_speed_margin_rpm - margin of toggle speed for high speed mode [rpm]
*                 f4_rpm_rad                     - rpm to rad
* Return Value  : None
***********************************************************************************************************************/
void mtr_set_param_encd_hs_sw_speed_margin_rad(mtr_encoder_highspeed_t *st_ehc,
                                               uint16_t f4_encd_hs_sw_speed_margin_rpm,
                                               float f4_rpm_rad);

/***********************************************************************************************************************
* Function Name : mtr_get_encd_tcnt
* Description   : Get encoder timer count
* Arguments     : u1_id - The motor ID which specify the motor connectors
* Return Value  : Encoder timer count
***********************************************************************************************************************/
uint16_t mtr_get_encd_tcnt(uint8_t u1_id);

/***********************************************************************************************************************
* Function Name : mtr_set_encd_tcnt
* Description   : Set encoder timer
* Arguments     : u1_id        - Motor ID
*                 u2_cnt_value - count
* Return Value  : None
***********************************************************************************************************************/
void mtr_set_encd_tcnt(uint8_t u1_id, uint16_t u2_cnt_value);

/***********************************************************************************************************************
* Function Name : mtr_encd_cnt_reset
* Description   : Encoder count reset
* Arguments     : st_ec - encoder structure (pointer)
*                 u1_id - The motor ID which specify the motor connectors
* Return Value  : None
***********************************************************************************************************************/
void mtr_encd_cnt_reset(mtr_encoder_t *st_ec, uint8_t u1_id);

/***********************************************************************************************************************
* Function Name : mtr_angle_adj_excit
* Description   : Angle adjust for forced excitation
* Arguments     : st_ec - encoder structure (pointer)
*                 u1_id - The motor ID which specify the motor connectors
* Return Value  : None
***********************************************************************************************************************/
void mtr_angle_adj_excit(mtr_encoder_t *st_ec, uint8_t u1_id);

/***********************************************************************************************************************
* Function Name : mtr_encd_hs_cnt_diff
* Description   : Calculate the difference of encoder count
* Arguments     : st_ehc - encoder highspeed structure (pointer)
*                 u1_id  - The motor ID which specify the motor connectors
* Return Value  : None 
***********************************************************************************************************************/
void mtr_encd_hs_cnt_diff(mtr_encoder_highspeed_t *st_ehc, uint8_t u1_id);

/***********************************************************************************************************************
* Function Name : mtr_encd_hs_speed_calc
* Description   : Calculate speed
* Arguments     : st_ehc         - encoder highspeed structure (pointer)
*                 f4_pos_err     - position error [rad]
*                 f4_ctrl_period - control period
* Return Value  : None 
***********************************************************************************************************************/
void mtr_encd_hs_speed_calc(mtr_encoder_highspeed_t *st_ehc, float f4_pos_err, float f4_ctrl_period);

/***********************************************************************************************************************
* Function Name : mtr_encd_hs_switch_encd_interrupt
* Description   : Swich encoder interrupt process
* Arguments     : st_ehc       - encoder highspeed structure (pointer)
*                 f4_speed_rad - speed [rad/s]
*                 u1_id        - The motor ID which specify the motor connectors
* Return Value  : None 
***********************************************************************************************************************/
void mtr_encd_hs_switch_encd_interrupt(mtr_encoder_highspeed_t *st_ehc, float f4_speed_rad, uint8_t u1_id);

/***********************************************************************************************************************
* Function Name : mtr_encd_hs_switch_ctrl_interrupt
* Description   : Switch carrier innterupt process
* Arguments     : st_ehc       - encoder highspeed structure (pointer)
*                 f4_speed_rad - speed [rad/s]
*                 u1_id        - The motor ID which specify the motor connectors
* Return Value  : None 
***********************************************************************************************************************/
void mtr_encd_hs_switch_ctrl_interrupt(mtr_encoder_highspeed_t *st_ehc, float f4_speed_rad, uint8_t u1_id);

#endif /* R_MTR_CTRL_ENCODER_H */
