/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products. No
* other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED. TO THE MAXIMUM
* EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES
* SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS
* SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability of
* this software. By using this software, you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2017 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/
/***********************************************************************************************************************
* File Name   : r_mtr_ctrl_hall.h
* Description : Definitions of a Hall control processes
***********************************************************************************************************************/
/**********************************************************************************************************************
* History : DD.MM.YYYY Version
*         : 05.04.2017 1.00
*         : 07.07.2017 1.01
***********************************************************************************************************************/

/* guard against multiple inclusion */
#ifndef R_MTR_CTRL_HALL_H
#define R_MTR_CTRL_HALL_H

/***********************************************************************************************************************
* Includes <System Includes> , "Project Includes"
***********************************************************************************************************************/
#include <stdint.h>

/***********************************************************************************************************************
* Macro definitions
***********************************************************************************************************************/
#define     MTR_HALL_EDGE_CW        (1)
#define     MTR_HALL_EDGE_CCW       (0)

/* Low Level = N pole , High Level = S pole */
#define     MTR_NL_SH               (1)    /* Enable */
/* How Level = N pole , Low Level = S pole */
#define     MTR_NH_SL               (0)    /* Disable */

#if MTR_NL_SH
#define     MTR_HALL_W              (1)
#define     MTR_HALL_UW             (5)
#define     MTR_HALL_U              (4)
#define     MTR_HALL_UV             (6)
#define     MTR_HALL_V              (2)
#define     MTR_HALL_VW             (3)
#elif MTR_NH_SL
#define     MTR_HALL_W              (6)
#define     MTR_HALL_UW             (2)
#define     MTR_HALL_U              (3)
#define     MTR_HALL_UV             (1)
#define     MTR_HALL_V              (5)
#define     MTR_HALL_VW             (4)
#endif

#define     MTR_HALL_EDGE_ANGLE_30  (30.0f*MTR_TWOPI_360)
#define     MTR_HALL_EDGE_ANGLE_90  (90.0f*MTR_TWOPI_360)
#define     MTR_HALL_EDGE_ANGLE_150 (150.0f*MTR_TWOPI_360)
#define     MTR_HALL_EDGE_ANGLE_210 (210.0f*MTR_TWOPI_360)
#define     MTR_HALL_EDGE_ANGLE_270 (270.0f*MTR_TWOPI_360)
#define     MTR_HALL_EDGE_ANGLE_330 (330.0f*MTR_TWOPI_360)
#define     MTR_HALL_EDGE_ERROR     (1)

/***********************************************************************************************************************
* Global structure
***********************************************************************************************************************/
typedef struct
{
    /* Hall */
    uint8_t       u1_hall_signal;                    /* Hall signal */
    uint8_t       u1_hall_pre_signal;                /* Previous value of hall signal */
    uint8_t       u1_hall_interrupt_flg;             /* Flag of hall interrupt */
    float         f4_hall_angle_rad;                 /* Using hall signal to set rotor initial angle [rad] */
} mtr_hall_t;

/***********************************************************************************************************************
* Global function definitions
***********************************************************************************************************************/

/***********************************************************************************************************************
* Function Name : mtr_hall_init
* Description   : Initialize variables when Hall control init
* Arguments     : st_hc              - Hall structure (pointer)
* Return Value  : None
***********************************************************************************************************************/
void mtr_hall_init(mtr_hall_t *st_hc);

/***********************************************************************************************************************
* Function Name : mtr_hall_reset
* Description   : Reset variables when Hall control
* Arguments     : st_hc              - Hall structure (pointer)
* Return Value  : None
***********************************************************************************************************************/
void mtr_hall_reset(mtr_hall_t *st_hc);

/***********************************************************************************************************************
* Function Name : mtr_angle_adj_hall_init
* Description   : Initialize rotor angle using hall sensor signal
* Arguments     : st_hc              - Hall structure (pointer)
* Return Value  : None
***********************************************************************************************************************/
float mtr_angle_adj_hall_init(mtr_hall_t *st_hc);

/***********************************************************************************************************************
* Function Name : mtr_get_hall_signal
* Description   : Process for Hall sensor interrupt
* Arguments     : u1_id - The motor ID which specify the motor connectors
* Return Value  : None
***********************************************************************************************************************/
uint8_t mtr_get_hall_signal(uint8_t u1_id);

/***********************************************************************************************************************
* Function Name : mtr_hall_angle_edge_set
* Description   : Calculate rotor angle when edge of hall signal is detected
* Arguments     : u1_hall_signal - Hall signal
                  u1_pre_hall_signal - previous value of Hall signal
* Return Value  : None
***********************************************************************************************************************/
float mtr_hall_angle_edge_set(uint8_t u1_hall_signal, uint8_t u1_pre_hall_signal);

/***********************************************************************************************************************
* Function Name : mtr_hall_angle_set
* Description   : Calculate rotor angle when the Hall sensor edge signal has not been detected yet
* Arguments     : u1_hall_signal - Hall signal
* Return Value  : None
***********************************************************************************************************************/
float mtr_hall_angle_set(uint8_t u1_hall_signal);

#endif /* R_MTR_CTRL_HALL_H */
