/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products. No
* other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED. TO THE MAXIMUM
* EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES
* SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS
* SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability of
* this software. By using this software, you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2017 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/
/***********************************************************************************************************************
* File Name    : r_mtr_mod.c
* Description  : This module convert 3-phase voltages to 3-phase duties
***********************************************************************************************************************/
/**********************************************************************************************************************
* History : DD.MM.YYYY Version
*         : 05.04.2017 1.00
*         : 07.07.2017 1.01
***********************************************************************************************************************/

/***********************************************************************************************************************
Includes   <System Includes> , "Project Includes"
***********************************************************************************************************************/
#include <stdint.h>
#include "r_mtr_mod.h"

/***********************************************************************************************************************
* Function Name: r_mod_svpwm
* Description  : Space vector modulation
* Arguments    : p_f4_v_in -
*                    Input data, in an array [Vu,Vv,Vw]
*                p_f4_v_out -
*                    Where to store output data, in an array [Vu,Vv,Vw]
* Return Value : None
***********************************************************************************************************************/
static inline r_mod_svpwm(const float *p_f4_v_in, float *p_f4_v_out)
{
    float f4_v_max;
    float f4_v_min;
    float f4_v_com;

    /* Sort vu vv vw */
    if (p_f4_v_in[0] > p_f4_v_in[1])
    {
        f4_v_max = p_f4_v_in[0];
        f4_v_min = p_f4_v_in[1];
    }
    else
    {
        f4_v_max = p_f4_v_in[1];
        f4_v_min = p_f4_v_in[0];
    }

    /* Vcom = (Vmin + Vmax)/2 */
    if (p_f4_v_in[2] > f4_v_max)
    {
        f4_v_com = (p_f4_v_in[2] + f4_v_min) * 0.5f;
    }
    else if (p_f4_v_in[2] < f4_v_min)
    {
        f4_v_com = (f4_v_max + p_f4_v_in[2]) * 0.5f;
    }
    else
    {
        f4_v_com = (f4_v_max + f4_v_min) * 0.5f;
    }

    p_f4_v_out[0] = p_f4_v_in[0] - f4_v_com;
    p_f4_v_out[1] = p_f4_v_in[1] - f4_v_com;
    p_f4_v_out[2] = p_f4_v_in[2] - f4_v_com;
} /* End of function r_mod_svpwm() */

/***********************************************************************************************************************
* Function Name: r_mod_limit
* Description  : Limits the duty cycle, and detect saturation (if function enabled)
* Arguments    : p_mod -
*                    Pointer to the modulation data structure
*                p_f4_duty -
*                    Pointer to the input and output duty cycle array, array length = 3
* Return Value : None
***********************************************************************************************************************/
static inline void r_mod_limit(mod_t *p_mod, float *p_f4_duty)
{
    float f4_min_duty = p_mod->f4_min_duty;
    float f4_max_duty = p_mod->f4_max_duty;

    if (p_f4_duty[0] > f4_max_duty)
    {
        p_f4_duty[0] = f4_max_duty;
#if (MOD_DETECT_SATURATION == 1)
        p_mod->u1_sat_flag |= MOD_SATFLAG_BITU;
#endif
    }
    else if (p_f4_duty[0] < f4_min_duty)
    {
        p_f4_duty[0] = f4_min_duty;
#if (MOD_DETECT_SATURATION == 1)
        p_mod->u1_sat_flag |= MOD_SATFLAG_BITU;
#endif
    }
    else
    {
        /* Clear correspond saturation flag bit */
#if (MOD_DETECT_SATURATION == 1)
        p_mod->u1_sat_flag &= (~MOD_SATFLAG_BITU);
#endif
    }
    if (p_f4_duty[1] > f4_max_duty)
    {
        p_f4_duty[1] = f4_max_duty;
#if (MOD_DETECT_SATURATION == 1)
        p_mod->u1_sat_flag |= MOD_SATFLAG_BITV;
#endif
    }
    else if (p_f4_duty[1] < f4_min_duty)
    {
        p_f4_duty[1] = f4_min_duty;
#if (MOD_DETECT_SATURATION == 1)
        p_mod->u1_sat_flag |= MOD_SATFLAG_BITV;
#endif
    }
    else
    {
        /* Clear correspond saturation flag bit */
#if (MOD_DETECT_SATURATION == 1)
        p_mod->u1_sat_flag &= (~MOD_SATFLAG_BITV);
#endif
    }
    if (p_f4_duty[2] > f4_max_duty)
    {
        p_f4_duty[2] = f4_max_duty;
#if (MOD_DETECT_SATURATION == 1)
        p_mod->u1_sat_flag |= MOD_SATFLAG_BITW;
#endif
    }
    else if (p_f4_duty[2] < f4_min_duty)
    {
        p_f4_duty[2] = f4_min_duty;
#if (MOD_DETECT_SATURATION == 1)
        p_mod->u1_sat_flag |= MOD_SATFLAG_BITW;
#endif
    }
    else
    {
        /* Clear correspond saturation flag bit */
#if (MOD_DETECT_SATURATION == 1)
        p_mod->u1_sat_flag &= (~MOD_SATFLAG_BITW);
#endif
    }
}/* End of function r_mod_limit() */

/***********************************************************************************************************************
* Function Name: R_MOD_Init
* Description  : Initializes modulation module
* Arguments    : p_mod -
*                    Pointer to the modulation data structure to be initialized
* Return Value : None
***********************************************************************************************************************/
void R_MOD_Init(mod_t *p_mod)
{
   if (p_mod)
   {
       p_mod->f4_1_div_vdc = 1.0f;                      /* Default value */
       p_mod->f4_max_duty = MOD_DEFAULT_MAX_DUTY;
       p_mod->f4_min_duty = 0.0f;
       p_mod->f4_neutral_duty = MOD_DEFAULT_MAX_DUTY * 0.5f;
       p_mod->f4_voltage_error_ratio = 0.0f;
       R_MOD_Reset(p_mod);
   }
} /* End of function R_MOD_Init() */

/***********************************************************************************************************************
* Function Name: R_MOD_Reset
* Description  : Resets modulation module
* Arguments    : p_mod -
*                    Pointer to the modulation data structure to be initialized
* Return Value : None
***********************************************************************************************************************/
void R_MOD_Reset(mod_t *p_mod)
{
    p_mod->u1_sat_flag = 0;
} /* End of function R_MOD_Reset() */

/***********************************************************************************************************************
* Function Name: R_MOD_Run
* Description  : Calculates duty cycle from input 3-phase voltage (bipolar)
* Arguments    : p_mod -
*                    Pointer to the modulation data structure
*              : p_f4_v_in -
*                    Pointer to the 3-phase input voltage
*              : p_f4_duty_out -
*                    Where to store the 3-phase output duty cycle
* Return Value : None
***********************************************************************************************************************/
void R_MOD_Run(mod_t *p_mod, const float *p_f4_v_in, float *p_f4_duty_out)
{
    float f4_v_out[3];

    if ((0 == p_mod) || (0 == p_f4_duty_out))
    {
        return;
    }

    r_mod_svpwm(p_f4_v_in, f4_v_out);

    p_f4_duty_out[0] = (f4_v_out[0] * p_mod->f4_1_div_vdc) + p_mod->f4_neutral_duty;
    p_f4_duty_out[1] = (f4_v_out[1] * p_mod->f4_1_div_vdc) + p_mod->f4_neutral_duty;
    p_f4_duty_out[2] = (f4_v_out[2] * p_mod->f4_1_div_vdc) + p_mod->f4_neutral_duty;

    r_mod_limit(p_mod, p_f4_duty_out);
} /* End of function R_MOD_Run() */

/***********************************************************************************************************************
* Function Name: R_MOD_RunUnipolar
* Description  : Calculates duty cycle from input 3-phase voltage (Unipolar)
* Arguments    : p_mod -
*                    Pointer to the modulation data structure
*              : p_f4_v_in -
*                    Pointer to the 3-phase input voltage
*              : p_f4_duty_out -
*                    Where to store the 3-phase output duty cycle
* Return Value : None
***********************************************************************************************************************/
void R_MOD_RunUnipolar(mod_t *p_mod, const float *p_f4_v_in, float *p_f4_duty_out)
{
    float f4_v_out[3];

    if ((0 == p_mod) || (0 == p_f4_duty_out))
    {
        return;
    }

#if (MOD_METHOD == MOD_METHOD_SVPWM)
    r_mod_svpwm(p_f4_v_in, f4_v_out);
#endif

    p_f4_duty_out[0] = (f4_v_out[0] * p_mod->f4_1_div_vdc);
    p_f4_duty_out[1] = (f4_v_out[1] * p_mod->f4_1_div_vdc);
    p_f4_duty_out[2] = (f4_v_out[2] * p_mod->f4_1_div_vdc);

    r_mod_limit(p_mod, p_f4_duty_out);
} /* End of function R_MOD_RunUnipolar() */

/***********************************************************************************************************************
* Function Name: R_MOD_GetVdc
* Description  : Gets the DC bus voltage used in modulation
* Arguments    : p_mod -
*                    Pointer to the modulation data structure
* Return Value : DC bus voltage
***********************************************************************************************************************/
float R_MOD_GetVdc(mod_t *p_mod)
{
    return (p_mod->f4_vdc);
} /* End of function R_MOD_GetVdc() */

/***********************************************************************************************************************
* Function Name: R_MOD_SetVdc
* Description  : Sets the DC bus voltage used in modulation
* Arguments    : p_mod -
*                    Pointer to the modulation data structure
*                f4_vdc -
*                    The DC bus voltage to be used in modulation
* Return Value : None
***********************************************************************************************************************/
void R_MOD_SetVdc(mod_t *p_mod, float f4_vdc)
{
    if (f4_vdc > 0.0f)
    {
        p_mod->f4_vdc = f4_vdc;
        p_mod->f4_1_div_vdc = 1.0f / f4_vdc;
    }
} /* End of function R_MOD_SetVdc() */

/***********************************************************************************************************************
* Function Name: R_MOD_GetMaxDuty
* Description  : Gets the maximum duty cycle
* Arguments    : p_mod -
*                    Pointer to the modulation data structure
* Return Value : Maximum duty cycle
***********************************************************************************************************************/
float R_MOD_GetMaxDuty(mod_t *p_mod)
{
    if (p_mod)
    {
       return (p_mod->f4_max_duty);
    }
    else
    {
        return 0.0f;
    }
} /* End of function R_MOD_GetMaxDuty() */

/***********************************************************************************************************************
* Function Name: R_MOD_SetMaxDuty
* Description  : Sets the maximum duty cycle
* Arguments    : p_mod -
*                    Pointer to the modulation data structure
*                f4_max_duty -
*                    Maximum duty cycle to set
* Return Value : None
***********************************************************************************************************************/
void R_MOD_SetMaxDuty(mod_t *p_mod, float f4_max_duty)
{
    if ((0 != p_mod) && ((f4_max_duty > 0.0f) && (f4_max_duty <= 1.0f)))
    {
       p_mod->f4_max_duty = f4_max_duty;
       p_mod->f4_neutral_duty = (p_mod->f4_max_duty + p_mod->f4_min_duty) * 0.5f;
    }
} /* End of function R_MOD_SetMaxDuty() */

/***********************************************************************************************************************
* Function Name: R_MOD_GetMinDuty
* Description  : Gets the minimum duty cycle
* Arguments    : p_mod -
*                    Pointer to the modulation data structure
* Return Value : minimum duty cycle
***********************************************************************************************************************/
float R_MOD_GetMinDuty(mod_t *p_mod)
{
    if (p_mod)
    {
       return (p_mod->f4_min_duty);
    }
    else
    {
        return 0.0f;
    }
} /* End of function R_MOD_GetMinDuty() */

/***********************************************************************************************************************
* Function Name: R_MOD_SetMinDuty
* Description  : Sets the minimum duty cycle
* Arguments    : p_mod -
*                    Pointer to the modulation data structure
*                f4_max_duty -
*                    Minimum duty cycle to set
* Return Value : None
***********************************************************************************************************************/
void R_MOD_SetMinDuty(mod_t *p_mod, float f4_min_duty)
{
    if ((0 != p_mod) && (f4_min_duty > 0.0f))
    {
       p_mod->f4_min_duty = f4_min_duty;
       p_mod->f4_neutral_duty = (p_mod->f4_max_duty + p_mod->f4_min_duty) * 0.5f;
    }
} /* End of function R_MOD_SetMinDuty() */

/***********************************************************************************************************************
* Function Name: R_MOD_SetVoltageErrorRatio
* Description  : Sets the voltage error ratio (VoltageError/Vdc)
* Arguments    : p_mod -
*                    Pointer to the modulation data structure
*                f4_voltage_error_ratio -
*                    The voltage error ratio := (VoltageError/Vdc), can use the (Dead-time*CarrierFreq) approximately
* Return Value : None
***********************************************************************************************************************/
void R_MOD_SetVoltageErrorRatio(mod_t *p_mod, float f4_voltage_error_ratio)
{
       p_mod->f4_voltage_error_ratio = f4_voltage_error_ratio;
} /* End of function R_MOD_SetVoltageErrorRatio() */

/***********************************************************************************************************************
* Function Name: R_MOD_GetSaturationFlag
* Description  : Gets saturation flag
* Arguments    : p_mod -
*                    Pointer to the modulation data structure
* Return Value : Saturation flag
*                   BIT0 - Phase U
*                   BIT1 - Phase V
*                   BIT2 - Phase W
*                   BIT3-7 - Reserved
***********************************************************************************************************************/
float R_MOD_GetSaturationFlag(mod_t *p_mod)
{
#if (MOD_DETECT_SATURATION == 1)
    if (p_mod)
    {
        return (p_mod->u1_sat_flag);
    }
    else
    {
        return 0;
    }
#else
    /* When saturation detection disabled, always return 0 */
    return 0;
#endif
} /* End of function R_MOD_GetSaturationFlag() */

/***********************************************************************************************************************
* Function Name: R_MOD_GetVoltageMultiplier
* Description  : Gets the voltage multiplier
* Arguments    : p_mod -
*                    Pointer to the modulation data structure
* Return Value : Voltage multiplier
***********************************************************************************************************************/
float R_MOD_GetVoltageMultiplier(mod_t *p_mod)
{
    float f4_usable_duty_cycle;

    f4_usable_duty_cycle = (p_mod->f4_max_duty - p_mod->f4_min_duty) - (2.0f * p_mod->f4_voltage_error_ratio) ;

#if (MOD_METHOD == MOD_METHOD_SPWM)
    return (f4_usable_duty_cycle);
#elif (MOD_METHOD == MOD_METHOD_SVPWM)
    return (f4_usable_duty_cycle * MOD_SVPWM_MULT);
#endif
} /* End of function R_MOD_GetVoltageMultiplier */

/***********************************************************************************************************************
* Function Name: R_MOD_GetVamax
* Description  : Gets the maximum magnitude of voltage vector
* Arguments    : p_mod -
*                    The pointer to the modulation data structure
* Return Value : The maximum magnitude of voltage vector
***********************************************************************************************************************/
float R_MOD_GetVamax(mod_t *p_mod)
{
    return ((MOD_VDC_TO_VAMAX_MULT * p_mod->f4_vdc) * R_MOD_GetVoltageMultiplier(p_mod));
} /* End of function R_MOD_GetVamax */
