/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products. No
* other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED. TO THE MAXIMUM
* EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES
* SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS
* SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability of
* this software. By using this software, you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2017 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/
/***********************************************************************************************************************
* File Name   : r_mtr_foc_current.c
* Description : The FOC current control module
***********************************************************************************************************************/
/**********************************************************************************************************************
* History : DD.MM.YYYY Version
*         : 05.04.2017 1.00
*         : 07.07.2017 1.01
***********************************************************************************************************************/

/***********************************************************************************************************************
* Includes <System Includes> , "Project Includes"
***********************************************************************************************************************/
/* Main associated header file */
#include "r_mtr_foc_current.h"

/* Project headers */
#include "r_mtr_ctrl_mcu.h"
#include "r_mtr_parameter.h"

/***********************************************************************************************************************
* Function Name : mtr_foc_current_init
* Description   : Initializes FOC current control module
* Arguments     : st_cc - The pointer to the FOC current control structure
* Return Value  : None
***********************************************************************************************************************/
void mtr_foc_current_init(mtr_current_control_t *st_cc)
{
    st_cc->f4_ctrl_period           = MTR_CTRL_PERIOD;
    st_cc->f4_lim_iq                = MTR_LIMIT_IQ;
    st_cc->st_pi_id.f4_ilimit       = MTR_I_LIMIT_VD;
    st_cc->st_pi_iq.f4_ilimit       = MTR_I_LIMIT_VQ;
    mtr_foc_current_reset(st_cc);
} /* End of function mtr_foc_current_init */

/***********************************************************************************************************************
* Function Name : mtr_foc_current_reset
* Description   : Resets FOC current control module
* Arguments     : st_cc - The pointer to the FOC current control structure
* Return Value  : None
***********************************************************************************************************************/
void mtr_foc_current_reset(mtr_current_control_t *st_cc)
{
    st_cc->f4_vd_ref           = 0.0f;
    st_cc->f4_vq_ref           = 0.0f;
    st_cc->f4_id_ref           = 0.0f;
    st_cc->f4_iq_ref           = 0.0f;
    st_cc->f4_id_ad            = 0.0f;
    st_cc->f4_iq_ad            = 0.0f;
    st_cc->st_pi_id.f4_err     = 0.0f;
    st_cc->st_pi_iq.f4_err     = 0.0f;
    st_cc->st_pi_id.f4_refi    = 0.0f;
    st_cc->st_pi_iq.f4_refi    = 0.0f;
} /* End of function mtr_foc_current_reset */

/***********************************************************************************************************************
* Function Name : mtr_foc_current_decoupling
* Description   : Decoupling control, cancels the interference voltage due to the magnet and inductance
* Arguments     : st_cc        - The pointer to the FOC current control structure
*                 f4_speed_rad - The electrical speed [rad/s]
*                 p_mtr        - The pointer to the motor parameter data structure
* Return Value  : None
***********************************************************************************************************************/
void mtr_foc_current_decoupling(mtr_current_control_t *st_cc, float f4_speed_rad, const mtr_parameter_t *p_mtr)
{
    float f4_temp0;

    f4_temp0 = p_mtr->f4_mtr_lq * st_cc->f4_iq_ref;               /* Lq * Iq */
    f4_temp0 = f4_speed_rad * f4_temp0;                           /* Speed * Lq * Iq */
    st_cc->f4_vd_ref -= f4_temp0;                                 /* Vd - Speed * Lq * Iq */

    f4_temp0 = p_mtr->f4_mtr_ld * st_cc->f4_id_ref;               /* Ld * id */
    f4_temp0 = f4_temp0 + p_mtr->f4_mtr_m;                        /* Ld * Id + Flux */
    f4_temp0 = f4_speed_rad * f4_temp0;                           /* Speed * (Ld * id + Flux) */
    st_cc->f4_vq_ref += f4_temp0;                                 /* Vq + Speed*(Ld * id + Flux) */
} /* End of function mtr_foc_current_decoupling */

/***********************************************************************************************************************
* Function Name : mtr_foc_current_pi_ctrl
* Description   : Current PI control
*                 Calculates the output voltage vector from current vector command and actual current vector
* Arguments     : st_cc - The pointer to the FOC current control structure
* Return Value  : None
***********************************************************************************************************************/
void mtr_foc_current_pi_ctrl(mtr_current_control_t *st_cc)
{
    /* The d-axis */
    st_cc->st_pi_id.f4_err = st_cc->f4_id_ref - st_cc->f4_id_ad;
    st_cc->f4_vd_ref = mtr_pi_ctrl(&(st_cc->st_pi_id));

    /* The q-axis */
    st_cc->st_pi_iq.f4_err = st_cc->f4_iq_ref - st_cc->f4_iq_ad;
    st_cc->f4_vq_ref = mtr_pi_ctrl(&(st_cc->st_pi_iq));
} /* End of function mtr_foc_current_pi_ctrl */
