/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products. No
* other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED. TO THE MAXIMUM
* EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES
* SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS
* SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability of
* this software. By using this software, you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2017 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/
/***********************************************************************************************************************
* File Name   : r_mtr_driver_access.h
* Description : Definitions of accessing driver processes
***********************************************************************************************************************/
/**********************************************************************************************************************
* History : DD.MM.YYYY Version
*         : 05.04.2017 1.00
*         : 07.07.2017 1.01
***********************************************************************************************************************/

/* guard against multiple inclusion */
#ifndef MTR_DRIVER_ACCESS_H
#define MTR_DRIVER_ACCESS_H

/***********************************************************************************************************************
* Includes <System Includes> , "Project Includes"
***********************************************************************************************************************/
#include <stdint.h>
#include "r_mtr_config.h"
#include "r_mtr_parameter.h"

/***********************************************************************************************************************
* Macro definitions
***********************************************************************************************************************/
/* gain mode */
#define    MTR_GAIN_DESIGN_MODE    (0)        /* Use values in design_parameters to gains in PI controllers */
#define    MTR_GAIN_DIRECT_MODE    (1)        /* Access PI controllers directly */

#define    MTR_DISABLE_AUTO_SET    (0)        /* Commands update */
#define    MTR_ENABLE_AUTO_SET     (1)        /* Automatically trigger the commands update (every 1ms) */

/* Select control gain design mode as default method when GAIN_MODE is undefined */
#ifndef    GAIN_MODE
#define    GAIN_MODE               (MTR_GAIN_DESIGN_MODE)
#endif

/***********************************************************************************************************************
* Global structure
***********************************************************************************************************************/
/* control input structure */
typedef struct
{
    uint8_t     u1_user_mode;                 /* user mode (0:ICS 1:BOARD) */
    uint8_t     u1_direction;                 /* reference rotation direction */
    uint8_t     u1_ctrl_loop_mode_buff;       /* control loop select */
    uint8_t     u1_ctrl_method_mode;          /* control method select */
    uint8_t     u1_state_pos_ref;             /* position command value management */
    uint8_t     u1_encd_angle_adj_mode;       /* encoder angle adjustment mode */
    int16_t     s2_ref_position_deg;          /* reference position [degree] */
    int16_t     s2_ref_speed_rpm;             /* reference speed [rpm] (mechanical) */
    uint16_t    u2_min_speed_rpm;             /* minimum speed [rpm] (mechanical) */
    uint16_t    u2_max_speed_rpm;             /* maximum speed [rpm] (mechanical) */
    uint16_t    u2_speed_limit_rpm;           /* over speed limit [rpm] (mechanical) */
    uint16_t    u2_hs_change_speed_rpm;       /* toggle speed for high speed mode [rpm] (mechanical) */
    uint16_t    u2_hs_change_margin_rpm;      /* margin of toggle speed for high speed mode [rpm] (mechanical) */
    uint16_t    u2_pos_dead_band;             /* position dead-band */
    uint16_t    u2_pos_band_limit;            /* positioning band limit */
    uint16_t    u2_encd_cpr_mech;             /* encoder counts per revolution (mechanical) */
    uint16_t    u2_pos_interval_time;         /* interval time for reference position update */
    uint16_t    u2_offset_calc_time;          /* calculation time for current offset / 100 [us] */
    float       f4_accel_time;                /* acceleration time */
    float       f4_ol_ref_id;                 /* reference d-axis current  [A] */
    float       f4_id_up_time;                /* reference d-axis current adding time[ms] */
    float       f4_limit_speed_change;        /* reference speed max change limit [rad/s] */
    float       f4_ipd_pos_kp_ratio;          /* position proportional control gain ratio for I-PD control*/
    float       f4_ipd_speed_k_ratio;         /* speed gain ratio for I-PD control */
    float       f4_ipd_err_limit_1;           /* position error limit 1 */
    float       f4_ipd_err_limit_2;           /* position error limit 2 */
    mtr_parameter_t        st_motor;          /* motor parameters */
    mtr_design_parameter_t st_design_params;  /* design parameter */
    mtr_ctrl_gain_t        st_ctrl_gain;      /* control gain */
} mtr_ctrl_input_t;

/***********************************************************************************************************************
* Global function definitions
***********************************************************************************************************************/
/***********************************************************************************************************************
* Function Name : R_MTR_SetUserifMode
* Description   : Set User interface mode
* Arguments     : u1_user_mode - User interface mode
* Return Value  : None
***********************************************************************************************************************/
void R_MTR_SetUserifMode(uint8_t u1_user_mode);

/***********************************************************************************************************************
* Function Name : R_MTR_InitControl
* Description   : Set motor ID and INACTIVE EVENT
* Arguments     : u1_id - The motor ID which specify the motor connectors
* Return Value  : None
***********************************************************************************************************************/
void R_MTR_InitControl(uint8_t u1_id);

/***********************************************************************************************************************
* Function Name : R_MTR_ExecEvent
* Description   : Excecution Event
* Arguments     : u1_event - Event
                  u1_id - The motor ID which specify the motor connectors
* Return Value  : None
***********************************************************************************************************************/
void R_MTR_ExecEvent(uint8_t u1_event, uint8_t u1_id);

/***********************************************************************************************************************
* Function Name : R_MTR_ChargeCapacitor
* Description   : Wait for charging capacitor
* Arguments     : u1_id - The motor ID which specify the motor connectors
* Return Value  : None
***********************************************************************************************************************/
void R_MTR_ChargeCapacitor(uint8_t u1_id);

/***********************************************************************************************************************
* Function Name : R_MTR_GetLoopModeStatus
* Description   : Get loop mode status
* Arguments     : u1_id - The motor ID which specify the motor connectors
* Return Value  : None
***********************************************************************************************************************/
uint8_t R_MTR_GetLoopModeStatus(uint8_t u1_id);

/***********************************************************************************************************************
* Function Name : R_MTR_SetPositionStatus
* Description   : Set position status
* Arguments     : u1_pos_status - position status
* Return Value  : None
***********************************************************************************************************************/
void R_MTR_SetPositionStatus(uint8_t u1_pos_status);

/***********************************************************************************************************************
* Function Name : R_MTR_SetPosition
* Description   : Set position reference
* Arguments     : s2_ref_position - position reference [degree]
* Return Value  : None
***********************************************************************************************************************/
void R_MTR_SetPosition(int16_t s2_ref_position);

/***********************************************************************************************************************
* Function Name : R_MTR_GetPosition
* Description   : Get position [degree]
* Arguments     : u1_id - The motor ID which specify the motor connectors
* Return Value  : position [degree]
***********************************************************************************************************************/
int16_t R_MTR_GetPosition(uint8_t u1_id);

/***********************************************************************************************************************
* Function Name : R_MTR_GetPositioningFlag
* Description   : Get Positioning Flag
* Arguments     : u1_id - The motor ID which specify the motor connectors
* Return Value  : positioning flag
***********************************************************************************************************************/
uint8_t R_MTR_GetPositioningFlag(uint8_t u1_id);

/***********************************************************************************************************************
* Function Name : R_MTR_SetSpeed
* Description   : Set speed reference
* Arguments     : ref_speed - speed reference [rpm]
* Return Value  : None
***********************************************************************************************************************/
void R_MTR_SetSpeed(int16_t ref_speed);

/***********************************************************************************************************************
* Function Name : R_MTR_GetSpeed
* Description   : Get speed value
* Arguments     : u1_id - The motor ID which specify the motor connectors
* Return Value  : speed
***********************************************************************************************************************/
int16_t R_MTR_GetSpeed(uint8_t u1_id);

/***********************************************************************************************************************
* Function Name : R_MTR_SetDir
* Description   : Set rotation direction
* Arguments     : dir - rotational direction
*                 u1_id - The motor ID which specify the motor connectors
* Return Value  : None
***********************************************************************************************************************/
void R_MTR_SetDir(uint8_t dir, uint8_t u1_id);

/***********************************************************************************************************************
* Function Name : R_MTR_GetDir
* Description   : Get rotation direction
* Arguments     : u1_id - The motor ID which specify the motor connectors
* Return Value  : rotational direction
***********************************************************************************************************************/
uint8_t R_MTR_GetDir(uint8_t u1_id);

/***********************************************************************************************************************
* Function Name : R_MTR_GetStatus
* Description   : Get status of motor control system
* Arguments     : u1_id - The motor ID which specify the motor connectors
* Return Value  : motor status
***********************************************************************************************************************/
uint8_t R_MTR_GetStatus(uint8_t u1_id);

/***********************************************************************************************************************
* Function Name : R_MTR_InputBuffParamReset
* Description   : Reset input buffer variables when motor control reset
* Arguments     : u1_id - The motor ID which specify the motor connectors
* Return Value  : None
***********************************************************************************************************************/
void R_MTR_InputBuffParamReset(void);

/***********************************************************************************************************************
* Function Name : R_MTR_CtrlInput
* Description   : Store com_* variables to control input buffer
* Arguments     : Control input structure (pointer)
* Return Value  : None
***********************************************************************************************************************/
void R_MTR_CtrlInput(mtr_ctrl_input_t *st_ctrl_input);

/***********************************************************************************************************************
* Function Name : R_MTR_SetVariables
* Description   : Set control input buffer to motor control structure members
* Arguments     : None
* Return Value  : None
***********************************************************************************************************************/
void R_MTR_SetVariables(void);

/***********************************************************************************************************************
* Function Name : R_MTR_AutoSetVariables
* Description   : Auto set control input buffer to motor control structure members
* Arguments     : None
* Return Value  : None
***********************************************************************************************************************/
void R_MTR_AutoSetVariables(void);

/***********************************************************************************************************************
* Function Name : R_MTR_CtrlGainCalc
* Description   : motor control parameter & gain calculation
* Arguments     : None
* Return Value  : None
***********************************************************************************************************************/
void R_MTR_CtrlGainCalc(void);

/***********************************************************************************************************************
* Function Name : R_MTR_UpdatePolling
* Description   : Polling u1_trig_enable_write to update configurations and commands
* Arguments     : None
* Return Value  : None
***********************************************************************************************************************/
void R_MTR_UpdatePolling(void);

#endif /* MTR_DRIVER_ACCESS_H */
