/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products. No
* other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED. TO THE MAXIMUM
* EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES
* SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS
* SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability of
* this software. By using this software, you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2017 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/
/***********************************************************************************************************************
* File Name   : r_mtr_ics.c
* Description : Processes of a user interface (tool)
***********************************************************************************************************************/
/**********************************************************************************************************************
* History : DD.MM.YYYY Version
*         : 05.04.2017 1.00
*         : 07.07.2017 1.01
***********************************************************************************************************************/

/***********************************************************************************************************************
* Includes <System Includes> , "Project Includes"
***********************************************************************************************************************/
/* Standard library headers */
#include <machine.h>

/* Main associated header file */
#include "r_mtr_ics.h"

/* Project headers */
#include "r_mtr_config.h"
#include "r_mtr_driver_access.h"

#if     defined(RX23T_MRSSK)
    #include "ICS_RX23T.h"
#elif   defined(RX24T_MRSSK)
    #include "ICS_RX24T.h"
#elif   defined(RX24U_MRSSK)
    #include "ICS_RX24U.h"
#else
    #error Please select a valid CPU board by defining a <MCU>_<MRSSK> macro in r_mtr_config.h
#endif

static uint8_t u1_cnt_ics;                         /* Counter for period of calling "scope_watchpoint" */
static mtr_ctrl_input_t  st_ctrl_input;            /* Structure for ICS input */

/***********************************************************************************************************************
* Exported global variables (to be accessed by other files)
***********************************************************************************************************************/
/* User command variables */
uint8_t      com_u1_direction;                     /* rotational direction (0:CW ,1:CCW) */
uint8_t      com_u1_ctrl_loop_mode;                /* loop mode select */
uint8_t      com_u1_ctrl_method_mode;              /* position loop method select */
uint8_t      com_u1_position_input_mode;           /* position command value management */
uint8_t      com_u1_encd_angle_adj_mode;           /* encoder angle adjustment mode */
int16_t      com_s2_ref_position_deg;              /* motor position reference [degree] (mechanical) */
int16_t      com_s2_ref_speed_rpm;                 /* motor speed reference [rpm] (mechanical) */
uint16_t     com_u2_min_speed_rpm;                 /* minimum speed [rpm] (mechanical) */
uint16_t     com_u2_max_speed_rpm;                 /* maximum speed [rpm] (mechanical) */
uint16_t     com_u2_overspeed_limit_rpm;           /* over speed limit [rpm] (mechanical) */
uint16_t     com_u2_hs_change_speed_rpm;           /* toggle speed for high speed mode [rpm] (mechanical) */
uint16_t     com_u2_hs_change_margin_rpm;          /* margin of toggle speed for high speed mode [rpm] (mechanical) */
uint16_t     com_u2_pos_interval_time;             /* interval time for reference position update */
uint16_t     com_u2_pos_dead_band;                 /* position dead-band */
uint16_t     com_u2_pos_band_limit;                /* positioning band limit */
uint16_t     com_u2_encd_cpr_mech;                 /* encoder counts per revolution (mechanical) */
uint16_t     com_u2_offset_calc_time;              /* current offset calculation time */
uint16_t     com_u2_mtr_pp;                        /* pole pairs */
float        com_f4_mtr_r;                         /* resistance [ohm] */
float        com_f4_mtr_ld;                        /* d-axis inductance [H] */
float        com_f4_mtr_lq;                        /* q-axis inductance [H] */
float        com_f4_mtr_m;                         /* permanent magnetic flux [Wb] */
float        com_f4_mtr_j;                         /* rotor inertia [kgm^2] */
float        com_f4_nominal_current_rms;           /* The nominal current[Arms] */
float        com_f4_current_omega;                 /* natural frequency for current loop [Hz] */
float        com_f4_current_zeta;                  /* damping ratio for current loop */
float        com_f4_speed_omega;                   /* natural frequency for speed loop [Hz] */
float        com_f4_speed_zeta;                    /* damping ratio for speed loop */
float        com_f4_pos_omega;                     /* natural frequency for position [Hz] */
float        com_f4_sob_omega;                     /* natural frequency for speed observer [Hz] */
float        com_f4_sob_zeta;                      /* damping ratio for speed observer */
float        com_f4_id_kp;                         /* d-axis current PI proportional gain */
float        com_f4_id_ki;                         /* d-axis current PI integral gain */
float        com_f4_iq_kp;                         /* q-axis current PI proportional gain */
float        com_f4_iq_ki;                         /* q-axis current PI integral gain */
float        com_f4_speed_kp;                      /* speed PI proportional gain */
float        com_f4_speed_ki;                      /* speed PI integral gain */
float        com_f4_pos_kp;                        /* position P proportional gain */
float        com_f4_ipd_speed_k_ratio;             /* speed gain ratio for I-PD */
float        com_f4_ipd_pos_kp_ratio;              /* position control gain ratio for I-PD */
float        com_f4_ipd_err_limit_1;               /* position error limit 1 */
float        com_f4_ipd_err_limit_2;               /* position error limit 2 */
float        com_f4_accel_time;                    /* reference acceleration time */
float        com_f4_ol_ref_id;                     /* id reference when open loop [A] */
float        com_f4_id_up_time;                    /* time to increase id */
float        com_f4_limit_speed_change;            /* limit of speed change */
uint8_t      com_u1_enable_write;                  /* ICS write enable flag */
uint8_t      g_u1_enable_write;                    /* ICS write enable flag */

/***********************************************************************************************************************
* Function Name : mtr_set_com_variables
* Description   : Set com variables to "tool" input buffer
* Arguments     : None
* Return Value  : None
***********************************************************************************************************************/
void mtr_set_com_variables(void)
{
    float f4_temp0;

    /**************************/
    /*    limit variables     */
    /**************************/
    com_s2_ref_speed_rpm = R_MTR_Limit(com_s2_ref_speed_rpm, com_u2_max_speed_rpm, com_u2_min_speed_rpm);
    /* current natural frequency limit 1Hz ~ 1000Hz */
    com_f4_current_omega = R_MTR_Limitf(com_f4_current_omega, MTR_MAX_CURRENT_OMEGA, MTR_MIN_POS_OMEGA);
    /* natural frequency & damping ratio for speed control loop */
    f4_temp0 = com_f4_current_omega / MTR_FREQ_BAND_LIMIT;
    if (f4_temp0 < com_f4_speed_omega)
    {
        com_f4_speed_omega = f4_temp0;
        /* speed natural frequency minimum limit 1Hz */
        com_f4_speed_omega = R_MTR_LowerLimitf(com_f4_speed_omega, MTR_MIN_POS_OMEGA);
    }
    /* natural frequency for position control loop */
    if (MTR_CTRL_PID == com_u1_ctrl_method_mode)
    {
        f4_temp0 = com_f4_speed_omega / MTR_FREQ_BAND_LIMIT;
        if (f4_temp0 < com_f4_pos_omega)
        {
            com_f4_pos_omega = f4_temp0;
            /* position natural frequency minimum limit 1Hz */
            com_f4_pos_omega = R_MTR_LowerLimitf(com_f4_pos_omega, MTR_MIN_POS_OMEGA);
        }
    }
    else if (MTR_CTRL_IPD == com_u1_ctrl_method_mode)
    {
        f4_temp0 = com_f4_current_omega / MTR_FREQ_BAND_LIMIT;
        if (f4_temp0 < com_f4_ipd_speed_k_ratio * com_f4_pos_omega)
        {
            com_f4_ipd_speed_k_ratio = MTR_FREQ_BAND_LIMIT;
            com_f4_pos_omega = f4_temp0 / com_f4_ipd_speed_k_ratio;
            /* position natural frequency minimum limit 1Hz */
            com_f4_pos_omega = R_MTR_LowerLimitf(com_f4_pos_omega, MTR_MIN_POS_OMEGA);
        }
    }
    else
    {
        /* Do Nothing */
    }

    /***** When com_s2_enable_sw and g_u1_enable_write are same value, rewrite enable. *****/
    if (com_u1_enable_write == g_u1_enable_write)
    {
        /* rotation direction */
        if (R_MTR_GetDir(MTR_ID_A) != com_u1_direction)
        {
            if (MTR_CCW < com_u1_direction)
            {
                com_u1_direction = MTR_CCW;
            }
        }
        st_ctrl_input.u1_direction = com_u1_direction;
        /* control loop select */
        st_ctrl_input.u1_ctrl_loop_mode_buff = com_u1_ctrl_loop_mode;
        /* control method select */
        st_ctrl_input.u1_ctrl_method_mode = com_u1_ctrl_method_mode;
        /* position command value management */
        st_ctrl_input.u1_state_pos_ref = com_u1_position_input_mode;
        /* encoder angle adjustment mode */
        st_ctrl_input.u1_encd_angle_adj_mode = com_u1_encd_angle_adj_mode;
        /* reference position [degree] (mechanical) */
        st_ctrl_input.s2_ref_position_deg = com_s2_ref_position_deg;
        /* reference speed [rpm] (mechanical) */
        st_ctrl_input.s2_ref_speed_rpm = com_s2_ref_speed_rpm;
        /* reference minimum speed [rpm] (mechanical) */
        st_ctrl_input.u2_min_speed_rpm = com_u2_min_speed_rpm;
        /* reference maximum speed [rpm] (mechanical) */
        st_ctrl_input.u2_max_speed_rpm = com_u2_max_speed_rpm;
        /* over speed limit [rpm] (mechanical) */
        st_ctrl_input.u2_speed_limit_rpm = com_u2_overspeed_limit_rpm;
        /* toggle speed for high speed mode [rpm] */
        st_ctrl_input.u2_hs_change_speed_rpm = com_u2_hs_change_speed_rpm;
        /* margin of toggle speed for high speed mode [rpm] */
        st_ctrl_input.u2_hs_change_margin_rpm = com_u2_hs_change_margin_rpm;
        /* interval time for reference position update */
        st_ctrl_input.u2_pos_interval_time = com_u2_pos_interval_time;
        /* position dead-band */
        st_ctrl_input.u2_pos_dead_band = com_u2_pos_dead_band;
        /* positioning band limit */
        st_ctrl_input.u2_pos_band_limit = com_u2_pos_band_limit;
        /* encoder counts per revolution (mechanical) */
        st_ctrl_input.u2_encd_cpr_mech = com_u2_encd_cpr_mech;
        /* current offset calculation time */
        st_ctrl_input.u2_offset_calc_time = com_u2_offset_calc_time;
        /* motor parameters */
        st_ctrl_input.st_motor.u2_mtr_pp = com_u2_mtr_pp;
        st_ctrl_input.st_motor.f4_mtr_r  = com_f4_mtr_r;
        st_ctrl_input.st_motor.f4_mtr_ld = com_f4_mtr_ld;
        st_ctrl_input.st_motor.f4_mtr_lq = com_f4_mtr_lq;
        st_ctrl_input.st_motor.f4_mtr_m  = com_f4_mtr_m;
        st_ctrl_input.st_motor.f4_mtr_j  = com_f4_mtr_j;
        st_ctrl_input.st_motor.f4_nominal_current_rms = com_f4_nominal_current_rms;
        /* acceleration time */
        st_ctrl_input.f4_accel_time = com_f4_accel_time;
        /* id reference when low speed */
        st_ctrl_input.f4_ol_ref_id = com_f4_ol_ref_id;
        /* time to increase id */
        st_ctrl_input.f4_id_up_time = com_f4_id_up_time;
        /* limit of speed change */
        st_ctrl_input.f4_limit_speed_change = com_f4_limit_speed_change;
        /* natural frequency & damping ratio for control loop */
        st_ctrl_input.st_design_params.f4_current_omega = com_f4_current_omega;
        st_ctrl_input.st_design_params.f4_current_zeta  = com_f4_current_zeta;
        st_ctrl_input.st_design_params.f4_speed_omega   = com_f4_speed_omega;
        st_ctrl_input.st_design_params.f4_speed_zeta    = com_f4_speed_zeta;
        st_ctrl_input.st_design_params.f4_pos_omega     = com_f4_pos_omega;
        /* natural frequency & damping ratio for observer */
        st_ctrl_input.st_design_params.f4_sob_omega     = com_f4_sob_omega;
        st_ctrl_input.st_design_params.f4_sob_zeta      = com_f4_sob_zeta;
        /* control gain */
        st_ctrl_input.st_ctrl_gain.f4_id_kp    = com_f4_id_kp;
        st_ctrl_input.st_ctrl_gain.f4_id_ki    = com_f4_id_ki;
        st_ctrl_input.st_ctrl_gain.f4_iq_kp    = com_f4_iq_kp;
        st_ctrl_input.st_ctrl_gain.f4_iq_ki    = com_f4_iq_ki;
        st_ctrl_input.st_ctrl_gain.f4_speed_kp = com_f4_speed_kp;
        st_ctrl_input.st_ctrl_gain.f4_speed_ki = com_f4_speed_ki;
        st_ctrl_input.st_ctrl_gain.f4_pos_kp   = com_f4_pos_kp;
        /* position control gain ratio for I-PD */
        st_ctrl_input.f4_ipd_pos_kp_ratio = com_f4_ipd_pos_kp_ratio;
        /* speed gain ratio for I-PD */
        st_ctrl_input.f4_ipd_speed_k_ratio = com_f4_ipd_speed_k_ratio;
        /* position error limit */
        st_ctrl_input.f4_ipd_err_limit_1 = com_f4_ipd_err_limit_1;
        st_ctrl_input.f4_ipd_err_limit_2 = com_f4_ipd_err_limit_2;
        /* copy variables com to ICS input */
        R_MTR_CtrlInput(&st_ctrl_input);

        g_u1_enable_write ^= 1;                         /* change every time 0 and 1 */
    }
} /* End of function mtr_set_com_variables */

/***********************************************************************************************************************
* Function Name : mtr_ics_variables_init
* Description   : Initialize valiables for Analyzer interface
* Arguments     : None
* Return Value  : None
***********************************************************************************************************************/
void mtr_ics_variables_init(void)
{
    g_u1_enable_write            = 0;
    com_u1_direction             = MTR_CW;
    com_u1_ctrl_loop_mode        = LOOP_MODE;
    com_u1_ctrl_method_mode      = POS_CTRL_MODE;
    com_u1_position_input_mode   = MTR_POS_TRAPEZOID;
    com_u1_encd_angle_adj_mode   = ANGLE_ADJUST_MODE;
    com_s2_ref_position_deg      = 0;
    com_s2_ref_speed_rpm         = 0;
    com_u2_min_speed_rpm         = MTR_MIN_SPEED_RPM;
    com_u2_max_speed_rpm         = MTR_MAX_SPEED_RPM;
    com_u2_overspeed_limit_rpm       = MTR_SPEED_LIMIT_RPM;
    com_u2_hs_change_speed_rpm   = MTR_HS_CHANGE_RPM;
    com_u2_hs_change_margin_rpm  = MTR_HS_CHANGE_MARGIN_RPM;
    com_u2_offset_calc_time      = MTR_OFFSET_CALC_TIME;
    com_u2_pos_dead_band         = MTR_POS_DEAD_BAND;
    com_u2_pos_band_limit        = MTR_POS_BAND_LIMIT;
    com_u2_pos_interval_time     = MTR_POS_INTERVAL_TIME;
    com_u2_encd_cpr_mech         = MTR_ENCD_CPR_MECH;
    com_u2_mtr_pp                = MTR_POLE_PAIRS;
    com_f4_mtr_r                 = MTR_R;
    com_f4_mtr_ld                = MTR_LD;
    com_f4_mtr_lq                = MTR_LQ;
    com_f4_mtr_m                 = MTR_M;
    com_f4_mtr_j                 = MTR_J;
    com_f4_nominal_current_rms   = MTR_NOMINAL_CURRENT_RMS;
    com_f4_accel_time            = MTR_ACCEL_TIME;
    com_f4_ol_ref_id             = MTR_REF_ID;
    com_f4_id_up_time            = MTR_ID_UP_TIME;
    com_f4_limit_speed_change    = MTR_LIMIT_SPEED_CHANGE;
    com_f4_current_omega         = MTR_CURRENT_OMEGA;
    com_f4_current_zeta          = MTR_CURRENT_ZETA;
    com_f4_speed_omega           = MTR_SPEED_OMEGA;
    com_f4_speed_zeta            = MTR_SPEED_ZETA;
    com_f4_pos_omega             = MTR_POS_OMEGA;
    com_f4_sob_omega             = SOB_OMEGA;
    com_f4_sob_zeta              = SOB_ZETA;
    com_f4_id_kp                 = 0;
    com_f4_id_ki                 = 0;
    com_f4_iq_kp                 = 0;
    com_f4_iq_ki                 = 0;
    com_f4_speed_kp              = 0;
    com_f4_speed_ki              = 0;
    com_f4_pos_kp                = 0;
    com_f4_ipd_speed_k_ratio     = POS_SPEED_RATIO;
    com_f4_ipd_pos_kp_ratio      = POS_KP_RATIO;
    com_f4_ipd_err_limit_1       = POS_IPD_ERR_LIMIT_1;
    com_f4_ipd_err_limit_2       = POS_IPD_ERR_LIMIT_2;
    com_u1_enable_write          = 0;

    /* initialize member of ICS input structure */
    st_ctrl_input.u1_direction             = com_u1_direction;
    st_ctrl_input.u1_ctrl_loop_mode_buff   = com_u1_ctrl_loop_mode;
    st_ctrl_input.u1_ctrl_method_mode      = com_u1_ctrl_method_mode;
    st_ctrl_input.u1_state_pos_ref         = com_u1_position_input_mode;
    st_ctrl_input.u1_encd_angle_adj_mode   = com_u1_encd_angle_adj_mode;
    st_ctrl_input.s2_ref_position_deg      = com_s2_ref_position_deg;
    st_ctrl_input.s2_ref_speed_rpm         = com_s2_ref_speed_rpm;
    st_ctrl_input.u2_min_speed_rpm         = com_u2_min_speed_rpm;
    st_ctrl_input.u2_max_speed_rpm         = com_u2_max_speed_rpm;
    st_ctrl_input.u2_speed_limit_rpm       = com_u2_overspeed_limit_rpm;
    st_ctrl_input.u2_hs_change_speed_rpm   = com_u2_hs_change_speed_rpm;
    st_ctrl_input.u2_hs_change_margin_rpm  = com_u2_hs_change_margin_rpm;
    st_ctrl_input.u2_offset_calc_time      = com_u2_offset_calc_time;
    st_ctrl_input.u2_pos_dead_band         = com_u2_pos_dead_band;
    st_ctrl_input.u2_pos_band_limit        = com_u2_pos_band_limit;
    st_ctrl_input.u2_pos_interval_time     = com_u2_pos_interval_time;
    st_ctrl_input.u2_encd_cpr_mech         = com_u2_encd_cpr_mech;
    st_ctrl_input.f4_accel_time            = com_f4_accel_time;
    st_ctrl_input.f4_ol_ref_id             = com_f4_ol_ref_id;
    st_ctrl_input.f4_id_up_time            = com_f4_id_up_time;
    st_ctrl_input.f4_limit_speed_change    = com_f4_limit_speed_change;
    st_ctrl_input.f4_ipd_pos_kp_ratio      = com_f4_ipd_pos_kp_ratio;
    st_ctrl_input.f4_ipd_speed_k_ratio     = com_f4_ipd_speed_k_ratio;
    st_ctrl_input.f4_ipd_err_limit_1       = com_f4_ipd_err_limit_1;
    st_ctrl_input.f4_ipd_err_limit_2       = com_f4_ipd_err_limit_2;
    /* motor parameter */
    st_ctrl_input.st_motor.u2_mtr_pp       = com_u2_mtr_pp;
    st_ctrl_input.st_motor.f4_mtr_r        = com_f4_mtr_r;
    st_ctrl_input.st_motor.f4_mtr_ld       = com_f4_mtr_ld;
    st_ctrl_input.st_motor.f4_mtr_lq       = com_f4_mtr_lq;
    st_ctrl_input.st_motor.f4_mtr_m        = com_f4_mtr_m;
    st_ctrl_input.st_motor.f4_mtr_j        = com_f4_mtr_j;
    st_ctrl_input.st_motor.f4_nominal_current_rms = com_f4_nominal_current_rms;
    /* design parameter */
    st_ctrl_input.st_design_params.f4_current_omega = com_f4_current_omega;
    st_ctrl_input.st_design_params.f4_current_zeta  = com_f4_current_zeta;
    st_ctrl_input.st_design_params.f4_speed_omega   = com_f4_speed_omega;
    st_ctrl_input.st_design_params.f4_speed_zeta    = com_f4_speed_zeta;
    st_ctrl_input.st_design_params.f4_pos_omega     = com_f4_pos_omega;
    st_ctrl_input.st_design_params.f4_sob_omega     = com_f4_sob_omega;
    st_ctrl_input.st_design_params.f4_sob_zeta      = com_f4_sob_zeta;
    /* pi gain */
    st_ctrl_input.st_ctrl_gain.f4_id_kp    = com_f4_id_kp;
    st_ctrl_input.st_ctrl_gain.f4_id_ki    = com_f4_id_ki;
    st_ctrl_input.st_ctrl_gain.f4_iq_kp    = com_f4_iq_kp;
    st_ctrl_input.st_ctrl_gain.f4_iq_ki    = com_f4_iq_ki;
    st_ctrl_input.st_ctrl_gain.f4_speed_kp = com_f4_speed_kp;
    st_ctrl_input.st_ctrl_gain.f4_speed_ki = com_f4_speed_ki;
    st_ctrl_input.st_ctrl_gain.f4_pos_kp   = com_f4_pos_kp;
} /* End of function mtr_ics_variables_init */

/***********************************************************************************************************************
* Function Name : mtr_ics_interrupt_process
* Description   : MTU3_4 interrupt (Period: 100 [us])
* Arguments     : None
* Return Value  : None
***********************************************************************************************************************/
void mtr_ics_interrupt_process(void)
{
    u1_cnt_ics++;
    if (MTR_ICS_DECIMATION <= u1_cnt_ics)                      /* Decimation of ICS call */
    {
        u1_cnt_ics = 0;
        ics2_watchpoint();                                     /* Call ICS */
    }

    /* Update commands and configurations when trigger flag is set */
    R_MTR_UpdatePolling();
} /* End of function mtr_ics_interrupt_process */
