/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products. No
* other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED. TO THE MAXIMUM
* EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES
* SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS
* SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability of
* this software. By using this software, you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2017 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/
/***********************************************************************************************************************
* File Name   : r_mtr_ctrl_hall.c
* Description : Processes of a hall control
***********************************************************************************************************************/
/**********************************************************************************************************************
* History : DD.MM.YYYY Version
*         : 05.04.2017 1.00
*         : 07.07.2017 1.01
***********************************************************************************************************************/

/***********************************************************************************************************************
* Includes <System Includes> , "Project Includes"
***********************************************************************************************************************/
/* Standard library headers */
#include <stdint.h>

/* Main associated header file */
#include "r_mtr_ctrl_hall.h"

/* Main associated header file */
#include "iodefine.h"
#include "r_mtr_common.h"
#include "r_mtr_ctrl_mcu.h"

/***********************************************************************************************************************
* Function Name : mtr_hall_init
* Description   : Initialize variables when hall control init
* Arguments     : st_hc              - hall structure (pointer)
* Return Value  : None
***********************************************************************************************************************/
void mtr_hall_init(mtr_hall_t *st_hc)
{
    /* initialize of hall variables */
    st_hc->u1_hall_signal              = 0;
    st_hc->u1_hall_pre_signal          = 0;
    st_hc->u1_hall_interrupt_flg       = MTR_FLG_CLR;
    st_hc->f4_hall_angle_rad           = 0.0f;
} /* End of function mtr_hall_init */

/***********************************************************************************************************************
* Function Name : mtr_hall_reset
* Description   : Reset variables when hall control
* Arguments     : st_hc              - hall structure (pointer)
* Return Value  : None
***********************************************************************************************************************/
void mtr_hall_reset(mtr_hall_t *st_hc)
{
    mtr_hall_init(st_hc);
} /* End of function mtr_hall_reset */

/***********************************************************************************************************************
* Function Name : mtr_angle_adj_hall_init
* Description   : Initialize hall angle for angle adjust mode
* Arguments     : st_hc              - hall structure (pointer)
* Return Value  : rotor angle of signal detected for hall sensor
***********************************************************************************************************************/
float mtr_angle_adj_hall_init(mtr_hall_t *st_hc)
{
    st_hc->u1_hall_signal = mtr_get_hall_signal(MTR_ID_A);      /* Get hall signal */
    st_hc->u1_hall_pre_signal = st_hc->u1_hall_signal;
    st_hc->f4_hall_angle_rad = mtr_hall_angle_set(st_hc->u1_hall_pre_signal);
                                                                /* between the hall edge angle set */

    return (st_hc->f4_hall_angle_rad);
} /* End of function mtr_angle_adj_hall_init */

/***********************************************************************************************************************
* Function Name : mtr_get_hall_signal
* Description   : Process for hall sensor interrupt
* Arguments     : u1_id - The motor ID which specify the motor connectors
* Return Value  : Hall sensor signal
***********************************************************************************************************************/
uint8_t mtr_get_hall_signal(uint8_t u1_id)
{
    uint8_t u1_pulse_hall_u;
    uint8_t u1_pulse_hall_v;
    uint8_t u1_pulse_hall_w;
    uint8_t u1_hall_signal;

    switch (u1_id)
    {
        case MTR_ID_A:
            /*** get hall signal ***/
            u1_pulse_hall_u = MTR_PORT_HALL_U;
            u1_pulse_hall_v = MTR_PORT_HALL_V;
            u1_pulse_hall_w = MTR_PORT_HALL_W;
            u1_hall_signal  = (u1_pulse_hall_u << 2);
            u1_hall_signal |= (u1_pulse_hall_v << 1);
            u1_hall_signal |= u1_pulse_hall_w;
            break;

        default:
            /* Do Nothing */
        break;
    }

    return (u1_hall_signal);
} /* End of function mtr_get_hall_signal */

/***********************************************************************************************************************
* Function Name : mtr_hall_angle_edge_set
* Description   : Process for hall edge angle set
* Arguments     : u1_hall_signal - hall signal
                  u1_pre_hall_signal - previous value of hall signal
* Return Value  : Rotor angle of signal detected for hall sensor
***********************************************************************************************************************/
float mtr_hall_angle_edge_set(uint8_t u1_hall_signal, uint8_t u1_pre_hall_signal)
{
    float f4_hall_angle_rad;

    switch (u1_pre_hall_signal)
    {
        case MTR_HALL_W: /* (1) */
            if (MTR_HALL_VW == u1_hall_signal) /* (1 -> 3) */
            {
                f4_hall_angle_rad = MTR_HALL_EDGE_ANGLE_330;
            }
            else if (MTR_HALL_UW == u1_hall_signal) /* (1 -> 5) */
            {
                f4_hall_angle_rad = MTR_HALL_EDGE_ANGLE_30;
            }
            else
            {
                f4_hall_angle_rad = MTR_HALL_EDGE_ERROR;
            }
        break;

        case MTR_HALL_UW: /* (5) */
            if (MTR_HALL_W == u1_hall_signal) /* (5 -> 1) */
            {
                f4_hall_angle_rad = MTR_HALL_EDGE_ANGLE_30;
            }
            else if (MTR_HALL_U == u1_hall_signal) /* (5 -> 4) */
            {
                f4_hall_angle_rad = MTR_HALL_EDGE_ANGLE_90;
            }
            else
            {
                f4_hall_angle_rad = MTR_HALL_EDGE_ERROR;
            }
        break;

        case MTR_HALL_U: /* (4) */
            if (MTR_HALL_UW == u1_hall_signal) /* (4 -> 5) */
            {
                f4_hall_angle_rad = MTR_HALL_EDGE_ANGLE_90;
            }
            else if (MTR_HALL_UV == u1_hall_signal) /* (4 -> 6) */
            {
                f4_hall_angle_rad = MTR_HALL_EDGE_ANGLE_150;
            }
            else
            {
                f4_hall_angle_rad = MTR_HALL_EDGE_ERROR;
            }
        break;

        case MTR_HALL_UV: /* (6) */
            if (MTR_HALL_U == u1_hall_signal) /* (6 -> 4) */
            {
                f4_hall_angle_rad = MTR_HALL_EDGE_ANGLE_150;
            }
            else if (MTR_HALL_V == u1_hall_signal) /* (6 -> 2) */
            {
                f4_hall_angle_rad = MTR_HALL_EDGE_ANGLE_210;
            }
            else
            {
                f4_hall_angle_rad = MTR_HALL_EDGE_ERROR;
            }
        break;

        case MTR_HALL_V: /* (2) */
            if (MTR_HALL_UV == u1_hall_signal) /* (2 -> 6) */
            {
                f4_hall_angle_rad = MTR_HALL_EDGE_ANGLE_210;
            }
            else if (MTR_HALL_VW == u1_hall_signal) /* (2 -> 3) */
            {
                f4_hall_angle_rad = MTR_HALL_EDGE_ANGLE_270;
            }
            else
            {
                f4_hall_angle_rad = MTR_HALL_EDGE_ERROR;
            }
        break;

        case MTR_HALL_VW: /* (3) */
            if (MTR_HALL_V == u1_hall_signal) /* (3 -> 2) */
            {
                f4_hall_angle_rad = MTR_HALL_EDGE_ANGLE_270;
            }
            else if (MTR_HALL_W == u1_hall_signal) /* (3 -> 1) */
            {
                f4_hall_angle_rad = MTR_HALL_EDGE_ANGLE_330;
            }
            else
            {
                f4_hall_angle_rad = MTR_HALL_EDGE_ERROR;
            }
        break;

        default:
            f4_hall_angle_rad = MTR_HALL_EDGE_ERROR;
        break;
    }

    return (f4_hall_angle_rad);
} /* End of function mtr_hall_angle_edge_set */

/***********************************************************************************************************************
* Function Name : mtr_hall_angle_set
* Description   : Process for hall sensor interrupt
* Arguments     : u1_hall_signal - hall signal
* Return Value  : Rotor angle of signal detected for hall sensor
***********************************************************************************************************************/
float mtr_hall_angle_set(uint8_t u1_hall_signal)
{
    float f4_hall_angle_rad;

    switch (u1_hall_signal)
    {
        case MTR_HALL_W:  /* (1) */
            f4_hall_angle_rad = MTR_HALL_EDGE_ANGLE_30 - (30.0f * MTR_TWOPI_360);
        break;
        
        case MTR_HALL_UW: /* (5) */
            f4_hall_angle_rad = MTR_HALL_EDGE_ANGLE_90 - (30.0f * MTR_TWOPI_360);
        break;

        case MTR_HALL_U:  /* (4) */
            f4_hall_angle_rad = MTR_HALL_EDGE_ANGLE_150 - (30.0f* MTR_TWOPI_360);
        break;

        case MTR_HALL_UV: /* (6) */
            f4_hall_angle_rad = MTR_HALL_EDGE_ANGLE_210 - (30.0f * MTR_TWOPI_360);
        break;

        case MTR_HALL_V:  /* (2) */
            f4_hall_angle_rad = MTR_HALL_EDGE_ANGLE_270 - (30.0f * MTR_TWOPI_360);
        break;

        case MTR_HALL_VW: /* (3) */
            f4_hall_angle_rad = MTR_HALL_EDGE_ANGLE_330 - (30.0f * MTR_TWOPI_360);
        break;
        
        default:
            f4_hall_angle_rad = MTR_HALL_EDGE_ERROR;
        break;
    }

    return (f4_hall_angle_rad);
} /* End of function mtr_hall_angle_set */
