/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products. No
* other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED. TO THE MAXIMUM
* EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES
* SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS
* SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability of
* this software. By using this software, you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2017 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/
/***********************************************************************************************************************
* File Name   : r_mtr_foc_control_encd_position.c
* Description : The processes of motor control
***********************************************************************************************************************/
/**********************************************************************************************************************
* History : DD.MM.YYYY Version
*         : 05.04.2017 1.00
*         : 07.07.2017 1.01
***********************************************************************************************************************/

/***********************************************************************************************************************
* Includes <System Includes> , "Project Includes"
***********************************************************************************************************************/
/* Standard library headers */
#include <math.h>
#include <machine.h>

/* Main associated header file */
#include "iodefine.h"
#include "r_mtr_foc_control_encd_position.h"

/* Project headers */
#include "r_mtr_common.h"
#include "r_mtr_ctrl_mcu.h"

/***********************************************************************************************************************
* Function Name : mtr_foc_init
* Description   : Initializes FOC drive system with default configuration and specified motor ID
* Arguments     : st_foc - The pointer to the FOC data structure
*                 u1_id - The motor ID which specify the motor connectors
* Return Value  : None
***********************************************************************************************************************/
void mtr_foc_init(mtr_foc_control_t *st_foc, uint8_t u1_id)
{
    /* Initialize system state machine */
    mtr_statemachine_init(&st_foc->st_stm);

    /* Store the id to access HW resources */
    st_foc->u1_id = u1_id;

    /* Reset configurations to default */
    mtr_foc_motor_default_init(st_foc);
} /* End of function mtr_foc_init */

/***********************************************************************************************************************
* Function Name : mtr_foc_motor_default_init
* Description   : Initializes motor drive modules with default configuration
* Arguments     : st_foc - The pointer to the FOC data structure
* Return Value  : None
***********************************************************************************************************************/
void mtr_foc_motor_default_init(mtr_foc_control_t *st_foc)
{
    mtr_design_parameter_t st_default_parameters;

    st_default_parameters.f4_current_omega = MTR_CURRENT_OMEGA;
    st_default_parameters.f4_current_zeta  = MTR_CURRENT_ZETA;
    st_default_parameters.f4_speed_omega   = MTR_SPEED_OMEGA;
    st_default_parameters.f4_speed_zeta    = MTR_SPEED_ZETA;
    st_default_parameters.f4_pos_omega     = MTR_POS_OMEGA;
    st_default_parameters.f4_sob_omega     = SOB_OMEGA;
    st_default_parameters.f4_sob_zeta      = SOB_ZETA;

    /* Initialize member of motor parameter structure */
    st_foc->st_motor.u2_mtr_pp = MTR_POLE_PAIRS;
    st_foc->st_motor.f4_mtr_r  = MTR_R;
    st_foc->st_motor.f4_mtr_ld = MTR_LD;
    st_foc->st_motor.f4_mtr_lq = MTR_LQ;
    st_foc->st_motor.f4_mtr_m  = MTR_M;
    st_foc->st_motor.f4_mtr_j  = MTR_J;

    /* Initialize offset calibration */
    st_foc->u1_flag_offset_calc      = MTR_FLG_CLR;
    st_foc->u2_offset_calc_time      = MTR_OFFSET_CALC_TIME;
    st_foc->u2_cnt_adjust            = 0;
    st_foc->f4_offset_iu             = 0.0f;
    st_foc->f4_offset_iw             = 0.0f;
    st_foc->f4_sum_iu_ad             = 0.0f;
    st_foc->f4_sum_iw_ad             = 0.0f;

    if (MTR_FLG_SET != st_foc->u1_flag_charge_cap)
    {
        st_foc->u1_flag_charge_cap = MTR_FLG_CLR;
    }
    else
    {
        /* Do Nothing */
    }
    st_foc->u1_direction             = MTR_CW;

    st_foc->f4_overspeed_limit_rad   = MTR_OVERSPEED_LIMIT_RAD;
    st_foc->f4_overvoltage_limit     = MTR_OVERVOLTAGE_LIMIT;
    st_foc->f4_undervoltage_limit    = MTR_UNDERVOLTAGE_LIMIT;
    st_foc->u1_encd_angle_adj_mode   = ANGLE_ADJUST_MODE;
    st_foc->u1_ctrl_loop_mode        = LOOP_MODE;
    st_foc->u1_ctrl_method_mode      = POS_CTRL_MODE;
    st_foc->f4_max_speed_rc_fil_rpm  = MTR_RC_FIL_MAX_SPEED_RPM;
    st_foc->f4_id_up_step            = MTR_ID_UP_STEP;
    st_foc->f4_ol_ref_id             = MTR_REF_ID;
    st_foc->f4_overcurrent_limit     = MTR_OVERCURRENT_LIMIT;
    /* Current control parameter reset */
    mtr_foc_current_init(&st_foc->st_cc);

    /* The dq-axis current PI gains */
    mtr_foc_current_pi_gain_calc(
        &st_foc->st_motor,
        &st_default_parameters,
        &st_foc->st_cc,
        st_foc->st_cc.f4_ctrl_period
    );

    /* Speed control parameter reset */
    mtr_speed_control_init(&st_foc->st_sc);

    /* Speed PI gains */
    mtr_speed_pi_gain_calc(
        &st_foc->st_motor,
        &st_default_parameters,
        &st_foc->st_sc,
        st_foc->st_sc.f4_speed_ctrl_period
    );

    /* position control parameter reset */
    mtr_position_control_init(&st_foc->st_pc);

    /* position P gain */
    mtr_pos_p_gain_calc(
        &st_default_parameters, &st_foc->st_pc
    );

    /* ipd control */
    mtr_ipd_ctrl_init(&st_foc->st_ipd);

    /* I-PD control gain */
    mtr_ipd_ctrl_gain_calc(
        &st_foc->st_motor,
        &st_default_parameters,
        &st_foc->st_ipd,
        st_foc->st_sc.f4_speed_ctrl_period
    );

    /* Setup current limits by nominal current of motor */
    mtr_foc_set_nominal_current(st_foc, MTR_NOMINAL_CURRENT_RMS);

    /* Voltage error compensation*/
#if (USE_VOLT_ERR_COMP == 1)
    mtr_volt_err_comp_init(&st_foc->st_volt_comp_p);

    /* Voltage error compensation enable */
    st_foc->st_volt_comp_p.u1_volt_err_comp_enable = MTR_VOLT_ERR_COMP_ENABLE;
#endif



    /* rotor angle parameter reset */
    mtr_rotor_angle_init(&st_foc->st_rotor_angle);
    /* encoder control parameter reset */
    mtr_encoder_init(&st_foc->st_ec);
    /* encoder high speed control parameter reset */
    mtr_encoder_highspeed_init(&st_foc->st_ehc);
    /* hall control parameter reset */
    mtr_hall_init(&st_foc->st_hc);
    /* position profiling parameter reset */
    mtr_position_profiling_init(&st_foc->st_ppf);
    /* speed observer */
    mtr_speed_observer_init(&st_foc->st_sob);
    /* speed observer */
    mtr_speed_observer_gain_calc(&st_foc->st_sob,
                                 &st_default_parameters,
                                 st_foc->st_sc.f4_speed_ctrl_period);

    /* Initialize modulation module */
    R_MOD_Init(&st_foc->st_mod);
    R_MOD_SetVdc(&st_foc->st_mod, MTR_INPUT_V);
    R_MOD_SetMaxDuty(&st_foc->st_mod, MTR_MAX_DUTY_CYCLE);
    R_MOD_SetMinDuty(&st_foc->st_mod, MTR_MIN_DUTY_CYCLE);
#if (USE_VOLT_ERR_COMP == 1)
    R_MOD_SetVoltageErrorRatio(&st_foc->st_mod, MTR_DUTY_CYCLE_DEADTIME);
#endif
    /* Reset motor */
    mtr_foc_motor_reset(st_foc);
} /* End of function mtr_foc_motor_default_init */

/***********************************************************************************************************************
* Function Name : mtr_foc_motor_reset
* Description   : Resets motor drive modules, configurations will not be reset
* Arguments     : st_foc - The pointer to the FOC data structure
* Return Value  : None
***********************************************************************************************************************/
void mtr_foc_motor_reset(mtr_foc_control_t *st_foc)
{
    /* Reset status structure members */
    st_foc->u2_run_mode              = MTR_MODE_INIT;
    st_foc->u2_ctrl_conf             = MTR_CONTROL_CURRENT;
    st_foc->u2_error_status          = MTR_ERROR_NONE;
    st_foc->u1_state_id_ref          = MTR_ID_UP;
    st_foc->u1_state_iq_ref          = MTR_IQ_ZERO_CONST;
    st_foc->u1_state_speed_ref       = MTR_SPEED_ZERO_CONST;
    st_foc->u1_state_pos_ref         = MTR_POS_TRAPEZOID;
    if(MTR_ANGLE_ADJ_HALL == st_foc->u1_encd_angle_adj_mode)
    {
        st_foc->u1_state_id_ref = MTR_ID_ZERO_CONST;
    }
    st_foc->f4_vdc_ad                = 0.0f;
    st_foc->f4_refu                  = 0.0f;
    st_foc->f4_refv                  = 0.0f;
    st_foc->f4_refw                  = 0.0f;
    st_foc->f4_iu_ad                 = 0.0f;
    st_foc->f4_iv_ad                 = 0.0f;
    st_foc->f4_iw_ad                 = 0.0f;
    st_foc->f4_modu                  = 0.0f;
    st_foc->f4_modv                  = 0.0f;
    st_foc->f4_modw                  = 0.0f;

    /* rotor angle parameter set*/
    mtr_rotor_angle_reset(&st_foc->st_rotor_angle);

    /* Reset current control module */
    mtr_foc_current_reset(&st_foc->st_cc);

    /* Reset speed control module */
    mtr_speed_control_reset(&st_foc->st_sc);

    /* Reset position control module */
    mtr_position_control_reset(&st_foc->st_pc);
    /* Encoder control parameter reset */
    mtr_encoder_reset(&st_foc->st_ec);
    /* Encoder high speed control parameter reset */
    mtr_encoder_highspeed_reset(&st_foc->st_ehc);
    /* Hall control parameter reset */
    mtr_hall_reset(&st_foc->st_hc);
    /* Position profiling parameter set */
    mtr_position_profiling_reset(&st_foc->st_ppf);
#if (USE_VOLT_ERR_COMP == 1)
    /* Voltage error compensation */
    mtr_volt_err_comp_reset(&st_foc->st_volt_comp_p);
    /* voltage error compensation enable */
    st_foc->st_volt_comp_p.u1_volt_err_comp_enable = MTR_VOLT_ERR_COMP_ENABLE;
#endif
    /* speed observer */
    mtr_speed_observer_reset(&st_foc->st_sob);
    /* ipd control */
    mtr_ipd_ctrl_reset(&st_foc->st_ipd);
    /* mod */
    R_MOD_Reset(&st_foc->st_mod);
    /* encoder init */
    mtr_encd_cnt_reset(&st_foc->st_ec, st_foc->u1_id);
    mtr_encd_interrupt_enable(st_foc->u1_id);
} /* End of function mtr_foc_motor_reset */

/***********************************************************************************************************************
* Function Name : mtr_foc_error
* Description   : Sets error flags and trigger error event when u2_error_flag is not zero
* Arguments     : st_foc - The pointer to the FOC data structure
*                 u2_error_flag - The error flags of the errors to report
* Return Value  : None
***********************************************************************************************************************/
void mtr_foc_error(mtr_foc_control_t *st_foc, uint16_t u2_error_flag)
{
    if (MTR_ERROR_NONE != u2_error_flag)
    {
        st_foc->u2_error_status |= u2_error_flag;
        mtr_statemachine_event(&st_foc->st_stm, st_foc, MTR_EVENT_ERROR);
    }
} /* End of function mtr_foc_error */

/***********************************************************************************************************************
* Function Name : mtr_calib_current_offset
* Description   : Measures the current offset
* Arguments     : st_foc - The pointer to the FOC data structure
* Return Value  : None
***********************************************************************************************************************/
void mtr_calib_current_offset(mtr_foc_control_t *st_foc)
{
    float f4_neutral_duty;

    f4_neutral_duty = st_foc->st_mod.f4_neutral_duty;
    mtr_inv_set_uvw(f4_neutral_duty, f4_neutral_duty, f4_neutral_duty, st_foc->u1_id);

    /*==================================*/
    /* Current offset calibration       */
    /*==================================*/
    st_foc->u2_cnt_adjust++;
    if (st_foc->u2_offset_calc_time <= st_foc->u2_cnt_adjust)
    {
        st_foc->u1_flag_offset_calc = MTR_FLG_SET;
        st_foc->u2_cnt_adjust = st_foc->u2_offset_calc_time;
        st_foc->f4_offset_iu = st_foc->f4_sum_iu_ad / st_foc->u2_offset_calc_time;
        st_foc->f4_offset_iw = st_foc->f4_sum_iw_ad / st_foc->u2_offset_calc_time;
    }
    else
    {
        st_foc->f4_sum_iu_ad += st_foc->f4_iu_ad;
        st_foc->f4_sum_iw_ad += st_foc->f4_iw_ad;
    }
} /* End of function mtr_calib_current_offset */

/***********************************************************************************************************************
* Function Name : mtr_current_offset_adjustment
* Description   : Calibrates the current offset
* Arguments     : st_foc - The pointer to the FOC data structure
* Return Value  : None
***********************************************************************************************************************/
void mtr_current_offset_adjustment(mtr_foc_control_t *st_foc)
{
    /* Current offset adjustment */
    st_foc->f4_iu_ad = st_foc->f4_iu_ad - st_foc->f4_offset_iu;
    st_foc->f4_iw_ad = st_foc->f4_iw_ad - st_foc->f4_offset_iw;
    st_foc->f4_iv_ad = - (st_foc->f4_iu_ad + st_foc->f4_iw_ad);
} /* End of function mtr_current_offset_adjustment */

/***********************************************************************************************************************
* Function Name : mtr_encd_pos_speed_calc
* Description   : calculate rotor position and speed
* Arguments     : st_foc - The pointer to the FOC data structure
* Return Value  : None
***********************************************************************************************************************/
void mtr_encd_pos_speed_calc(mtr_foc_control_t *st_foc)
{
    uint16_t u2_temp;
    float    f4_temp, f4_temp_pos_err;

    if (MTR_ENCD_ANGLE_ADJ_FIN == st_foc->st_ec.u1_encd_angle_adj_status)
    {
        /* integrate encoder counter difference */
        mtr_encd_hs_cnt_diff(&st_foc->st_ehc, st_foc->u1_id);
        /*=================================*/
        /*    angle & speed calculation    */
        /*=================================*/
        f4_temp_pos_err = 0;
        if (MTR_LOOP_SPEED == st_foc->u1_ctrl_loop_mode)
        {
            /* limit count value to counts of one rotation */
            if (st_foc->st_ehc.s4_encd_hs_angle_cnt >= (int32_t)st_foc->st_ec.u2_encd_cpr_mech)
            {
                st_foc->st_ehc.s4_encd_hs_angle_cnt -= (int32_t)st_foc->st_ec.u2_encd_cpr_mech;
                f4_temp_pos_err += MTR_TWOPI;
            }
            else if (st_foc->st_ehc.s4_encd_hs_angle_cnt < 0.0f)
            {
                st_foc->st_ehc.s4_encd_hs_angle_cnt += (int32_t)st_foc->st_ec.u2_encd_cpr_mech;
                f4_temp_pos_err -= MTR_TWOPI;
            }
            else
            {
                /* Do Nothing */
            }
        }
        /* calculate angle (mechanical) */
        f4_temp = MTR_TWOPI * st_foc->st_ehc.s4_encd_hs_angle_cnt;
        st_foc->st_ehc.f4_encd_hs_pos_rad = f4_temp * st_foc->st_ec.f4_encd_cpr_mech_inv;
        /* speed calculation */
        f4_temp_pos_err += st_foc->st_ehc.f4_encd_hs_pos_rad - st_foc->st_ehc.f4_encd_hs_pos_pre_rad;
        mtr_encd_hs_speed_calc(&st_foc->st_ehc, f4_temp_pos_err, st_foc->st_cc.f4_ctrl_period);
clrpsw_i();
        /* integrate difference between previous MTU0.TCNT and current MTU0.TCNT */
        st_foc->st_ec.u2_encd_timer_cap_tcnt =  MTR_ENCD_PLSWIDTH_TCNT;
        u2_temp = (st_foc->st_ec.u2_encd_timer_cap_tcnt - st_foc->st_ec.u2_encd_timer_cap_pre_tcnt);
        st_foc->st_ec.s4_encd_pulse_width_buff += (int32_t)u2_temp;
        st_foc->st_ec.u2_encd_timer_cap_pre_tcnt = st_foc->st_ec.u2_encd_timer_cap_tcnt;
        /* zero speed detection */
        if (st_foc->st_ec.s4_encd_pulse_width_buff > MTR_ENCD_ZEROSPEED_PW)
        {
            st_foc->st_ec.s4_encd_pulse_width_buff = MTR_ENCD_ZEROSPEED_PW;
        }
setpsw_i();
    }
} /* End of function mtr_encd_pos_speed_calc */

/***********************************************************************************************************************
* Function Name : mtr_angle_speed
* Description   : Angle and speed detection
* Arguments     : st_foc - The pointer to the FOC data structure
* Return Value  : None
***********************************************************************************************************************/
void mtr_angle_speed(mtr_foc_control_t *st_foc)
{
    float f4_angle_rad = 0.0f;

    if(MTR_MODE_INIT == st_foc->u2_run_mode)
    {
        f4_angle_rad = st_foc->st_hc.f4_hall_angle_rad;
    }
    else if (MTR_MODE_BOOT == st_foc->u2_run_mode)
    {
        /* speed must be zero while adjusting angle */
        st_foc->st_ec.f4_encd_speed_rad = 0.0f;

        if (MTR_ENCD_ANGLE_ADJ_90DEG == st_foc->st_ec.u1_encd_angle_adj_status)
        {
            /* set alignment angle to Pi/2[rad] */
            f4_angle_rad = MTR_TWOPI * 0.25f;
        }
        else if (MTR_ENCD_ANGLE_ADJ_0DEG == st_foc->st_ec.u1_encd_angle_adj_status)
        {
            /* set alignment angle to 0[rad] */
            f4_angle_rad = 0.0f;
        }
        else
        {
                /* Do Nothing */
        }
    }
    else if (MTR_MODE_DRIVE == st_foc->u2_run_mode)
    {
        if (MTR_PS_CALC_CTRL_INTERRUPT == st_foc->st_ehc.u1_encd_pos_speed_calc_mode)
        {
            st_foc->st_sc.f4_speed_rad = st_foc->st_ehc.f4_encd_hs_speed_rad * st_foc->st_motor.u2_mtr_pp;
            st_foc->st_pc.f4_pos_rad = st_foc->st_ehc.f4_encd_hs_pos_rad;
            mtr_encd_hs_switch_encd_interrupt(&st_foc->st_ehc, st_foc->st_sc.f4_speed_rad, st_foc->u1_id);
        }
        else if (MTR_PS_CALC_ENCD_INTERRUPT == st_foc->st_ehc.u1_encd_pos_speed_calc_mode)
        {
            st_foc->st_sc.f4_speed_rad = st_foc->st_ec.f4_encd_speed_rad * st_foc->st_motor.u2_mtr_pp;
            st_foc->st_pc.f4_pos_rad = st_foc->st_ec.f4_encd_pos_rad;
            mtr_encd_hs_switch_ctrl_interrupt(&st_foc->st_ehc, st_foc->st_sc.f4_speed_rad, st_foc->u1_id);
        }

        /* convert mechanical angle to electrical angle */
        if (MTR_ANGLE_ADJ_HALL == st_foc->u1_encd_angle_adj_mode)
        {
            f4_angle_rad = st_foc->st_hc.f4_hall_angle_rad + fmod((st_foc->st_pc.f4_pos_rad * st_foc->st_motor.u2_mtr_pp), MTR_TWOPI);
        }
        else if (MTR_ANGLE_ADJ_EXCIT == st_foc->u1_encd_angle_adj_mode)
        {
            f4_angle_rad = fmod((st_foc->st_pc.f4_pos_rad * st_foc->st_motor.u2_mtr_pp), MTR_TWOPI);
        }
        else
        {
            /* Do Nothing */
        }
    }
    else
    {
        /* Do Nothing */
    }

    if (f4_angle_rad >= MTR_TWOPI)
    {
        f4_angle_rad = f4_angle_rad - MTR_TWOPI;
    }
    else if (f4_angle_rad < 0.0f)
    {
        f4_angle_rad = f4_angle_rad + MTR_TWOPI;
    }
    else
    {
        /* Do nothing */
    }
    /* Update rotor angle */
    mtr_rotor_angle_update(&st_foc->st_rotor_angle, f4_angle_rad);
} /* End of function mtr_angle_speed */

/***********************************************************************************************************************
* Function Name : mtr_decide_direction
* Description   : Decide direction
* Arguments     : st_foc - The pointer to the FOC data structure
* Return Value  : None
***********************************************************************************************************************/
void mtr_decide_direction(mtr_foc_control_t *st_foc)
{
    if (0.0f <= st_foc->st_sc.f4_speed_rad)
    {
        st_foc->u1_direction = MTR_CW;
    }
    else if (0.0f > st_foc->st_sc.f4_speed_rad)
    {
        st_foc->u1_direction = MTR_CCW;
    }
    else
    {
        /* Do Nothing */
    }
} /* End of function mtr_decide_direction */


/***********************************************************************************************************************
* Function Name : mtr_set_speed_ref
* Description   : Updates the speed reference
* Arguments     : st_foc - The pointer to the FOC data structure
* Return Value  : Speed reference
***********************************************************************************************************************/
float mtr_set_speed_ref(mtr_foc_control_t *st_foc)
{
    float f4_speed_ref_calc_rad;
    float f4_speed_ff_rad;

    switch (st_foc->u1_state_speed_ref)
    {
        case MTR_SPEED_ZERO_CONST:
            /* speed must be zero while encoder angle adjustment */
            if (MTR_ENCD_ANGLE_ADJ_FIN == st_foc->st_ec.u1_encd_angle_adj_status)
            {
                /* check control loop mode */
                if (MTR_LOOP_SPEED == st_foc->u1_ctrl_loop_mode)
                {
                    st_foc->u1_state_speed_ref = MTR_SPEED_CHANGE;
                }
                else if (MTR_LOOP_POSITION == st_foc->u1_ctrl_loop_mode)
                {
                    st_foc->u1_state_speed_ref = MTR_POS_CONTROL_OUTPUT;
                }
                else
                {
                    /* Do Nothing */
                }
            }
        break;

        case MTR_POS_CONTROL_OUTPUT:
            /* check control method mode */
            if (MTR_CTRL_PID == st_foc->u1_ctrl_method_mode)
            {
                st_foc->st_pc.f4_pos_err_rad = st_foc->st_pc.f4_ref_pos_rad_ctrl - st_foc->st_pc.f4_pos_rad;
                st_foc->st_pc.f4_pos_err_rad = mtr_position_deadband_set(st_foc, st_foc->st_pc.f4_pos_err_rad);
                f4_speed_ref_calc_rad = mtr_p_ctrl_pos(&st_foc->st_pc,
                                                       &st_foc->st_motor,
                                                       st_foc->st_pc.f4_pos_err_rad);
            }
            else if (MTR_CTRL_IPD == st_foc->u1_ctrl_method_mode)
            {
                st_foc->st_pc.f4_pos_err_rad = mtr_ipd_error_calc(&st_foc->st_ipd,
                                                                st_foc->st_pc.f4_pos_rad,
                                                                st_foc->st_pc.f4_ref_pos_rad_ctrl);
                st_foc->st_pc.f4_pos_err_rad = mtr_position_deadband_set(st_foc, st_foc->st_pc.f4_pos_err_rad);
                f4_speed_ref_calc_rad = mtr_ipd_ctrl(&st_foc->st_ipd,
                                                     &st_foc->st_motor,
                                                     st_foc->st_pc.f4_pos_err_rad,
                                                     st_foc->st_sc.f4_max_speed_rad);
            }
            else
            {
                /* Do Nothing */
            }
            /* Speed Reference Profiling mode */
            if (MTR_POS_TRAPEZOID == st_foc->u1_state_pos_ref)
            {
                /* speed feed-forward control */
                f4_speed_ff_rad = mtr_speed_profile_feed_forward(&st_foc->st_ppf);
                f4_speed_ref_calc_rad = mtr_speed_feed_foward_control(&st_foc->st_pc,
                                                                      f4_speed_ff_rad,
                                                                      f4_speed_ref_calc_rad,
                                                                      st_foc->st_motor.u2_mtr_pp);
            }
        break;

        case MTR_SPEED_CHANGE:
            f4_speed_ref_calc_rad = mtr_speed_rate_limit(&st_foc->st_sc);
        break;

        default:
            /* Do Nothing */
        break;
    }
    /* speed reference limit */
    f4_speed_ref_calc_rad = R_MTR_LimitfAbs(f4_speed_ref_calc_rad, st_foc->st_sc.f4_max_speed_rad);

    /* return speed reference */
    return (f4_speed_ref_calc_rad);
} /* End of function mtr_set_speed_ref */

/***********************************************************************************************************************
* Function Name : mtr_set_iq_ref
* Description   : Updates the q-axis current reference
* Arguments     : st_foc - The pointer to the FOC data structure
* Return Value  : Iq reference
***********************************************************************************************************************/
float mtr_set_iq_ref(mtr_foc_control_t *st_foc)
{
    float f4_iq_ref_calc;
    float f4_temp_speed_rad;

    switch (st_foc->u1_state_iq_ref)
    {
        case MTR_IQ_ZERO_CONST:
            /*** constant zero set ***/
            f4_iq_ref_calc = 0.0f;
            st_foc->u2_ctrl_conf = MTR_CONTROL_CURRENT;
            if (MTR_MODE_DRIVE == st_foc->u2_run_mode)
            {
                st_foc->u1_state_iq_ref = MTR_IQ_SPEED_PI_OUTPUT;
            }
        break;

        case MTR_IQ_SPEED_PI_OUTPUT:
            st_foc->u2_ctrl_conf |= MTR_CONTROL_SPEED + MTR_CONTROL_POSITION;
            /*** speed PI control ***/
            if (MTR_CTRL_PID == st_foc->u1_ctrl_method_mode)
            {
                f4_temp_speed_rad = mtr_speed_observer(&st_foc->st_sob,
                                                       &st_foc->st_motor,
                                                       st_foc->st_cc.f4_iq_ref,
                                                       st_foc->st_sc.f4_speed_rad);
                f4_iq_ref_calc = mtr_speed_pi_control(&st_foc->st_sc, f4_temp_speed_rad);
            }
            else if (MTR_CTRL_IPD == st_foc->u1_ctrl_method_mode)
            {
                f4_temp_speed_rad = mtr_speed_observer(&st_foc->st_sob,
                                                       &st_foc->st_motor,
                                                       st_foc->st_cc.f4_iq_ref,
                                                       st_foc->st_sc.f4_speed_rad);
                f4_iq_ref_calc = mtr_ipd_speed_p_ctrl(&st_foc->st_ipd,
                                                      st_foc->st_sc.f4_ref_speed_rad_ctrl,
                                                      f4_temp_speed_rad);
            }
            else
            {
                /* Do Nothing */
            }
            /*** iq reference limit ***/
            f4_iq_ref_calc = R_MTR_LimitfAbs(f4_iq_ref_calc, st_foc->st_cc.f4_lim_iq);
        break;

        default:
            /* Do Nothing */
        break;
    }

    /* return iq reference */
    return (f4_iq_ref_calc);
} /* End of function mtr_set_iq_ref */

/***********************************************************************************************************************
* Function Name : mtr_set_id_ref
* Description   : Updates the d-axis current reference
* Arguments     : st_foc - The pointer to the FOC data structure
* Return Value  : Id reference
***********************************************************************************************************************/
float mtr_set_id_ref(mtr_foc_control_t *st_foc)
{
    float f4_id_ref_calc;

    switch (st_foc->u1_state_id_ref)
    {
        case MTR_ID_UP:
            /* d axis current ramping */
            f4_id_ref_calc = st_foc->st_cc.f4_id_ref + st_foc->f4_id_up_step;
            if (f4_id_ref_calc >= st_foc->f4_ol_ref_id)
            {
                f4_id_ref_calc = st_foc->f4_ol_ref_id;
                st_foc->u1_state_id_ref = MTR_ID_CONST;
            }
        break;

        case MTR_ID_CONST:
            f4_id_ref_calc = st_foc->st_cc.f4_id_ref;
            st_foc->st_ec.u2_encd_angle_adj_cnt++;
            if (st_foc->st_ec.u2_encd_angle_adj_cnt >= st_foc->st_ec.u2_encd_angle_adj_time)
            {
                f4_id_ref_calc = 0.0f;
                /* angle adjusted to 90 degree */
                if (MTR_ENCD_ANGLE_ADJ_90DEG == st_foc->st_ec.u1_encd_angle_adj_status)
                {
                    /* repeat soft start */
                    st_foc->u1_state_id_ref = MTR_ID_UP;
                }
                /* angle adjusted to 0 degree */
                else if (MTR_ENCD_ANGLE_ADJ_0DEG == st_foc->st_ec.u1_encd_angle_adj_status)
                {
                    /* id mode transition to zero constant mode */
                    st_foc->u1_state_id_ref = MTR_ID_ZERO_CONST;
                }
                else
                {
                    /* Do Nothing */
                }
                mtr_angle_adj_excit(&st_foc->st_ec, st_foc->u1_id);
            }
        break;

        case MTR_ID_ZERO_CONST:
            /* id zero constant */
            f4_id_ref_calc = 0.0f;
        break;

        default:
            /* Do Nothing */
        break;
    }

    /* return id reference */
    return (f4_id_ref_calc);
} /* End of function mtr_set_id_ref */

/***********************************************************************************************************************
* Function Name : mtr_set_pos_ref
* Description   : Set position reference
* Arguments     : st_foc - The pointer to the FOC data structure
* Return Value  : position reference
***********************************************************************************************************************/
float mtr_set_pos_ref(mtr_foc_control_t *st_foc)
{
    float f4_ref_pos_rad_calc;

    switch (st_foc->u1_state_pos_ref)
    {
        case MTR_POS_CONST:
            /* position zero constant */
            f4_ref_pos_rad_calc = 0.0f;
            if (MTR_MODE_DRIVE == st_foc->u2_run_mode)
            {
                st_foc->u1_state_pos_ref = MTR_POS_TRAPEZOID;
            }
        break;

        case MTR_POS_STEP:
            /* update reference position */
            f4_ref_pos_rad_calc = st_foc->st_pc.f4_ref_pos_rad;
        break;

        case MTR_POS_TRAPEZOID:
            /* check reference position update */
            if (st_foc->st_pc.f4_ref_pos_pre_rad != st_foc->st_pc.f4_ref_pos_rad)
            {
                st_foc->st_ppf.f4_pos_st_rad = st_foc->st_pc.f4_ref_pos_pre_rad;
                st_foc->st_ppf.f4_pos_ed_rad = st_foc->st_pc.f4_ref_pos_rad;
                st_foc->st_pc.f4_ref_pos_pre_rad = st_foc->st_pc.f4_ref_pos_rad;
            }
        break;

        default:
            /* Do Nothing */
        break;
    }

    if(MTR_POS_TRAPEZOID == st_foc->u1_state_pos_ref)
    {
        /* profiling reference position (trapezoid control) */
        f4_ref_pos_rad_calc = mtr_position_profile(&st_foc->st_ppf);
    }

    /* return speed reference */
    return (f4_ref_pos_rad_calc);
} /* End of function mtr_set_pos_ref */

/***********************************************************************************************************************
* Function Name : mtr_position_deadband_set
* Description   : Set position dead-band
* Arguments     : st_foc - The pointer to the FOC data structure
* Return Value  : None
***********************************************************************************************************************/
float mtr_position_deadband_set(mtr_foc_control_t *st_foc, float f4_pos_err_rad)
{
    float f4_temp0;

    f4_temp0 = fabsf(f4_pos_err_rad);
    if ((float)st_foc->st_pc.u2_pos_dead_band * st_foc->st_ec.f4_encd_angle_diff >= f4_temp0)
    {
        f4_pos_err_rad = 0.0f;
    }

    return (f4_pos_err_rad);
} /* End of function mtr_position_deadband_set */

/***********************************************************************************************************************
* Function Name : mtr_check_over_speed_error
* Description   : Checks over-speed error
* Arguments     : f4_speed_rad - The electrical speed[rad/s]
*                 f4_speed_limit_rad - The speed[rad/s] threshold of the over-speed error, should be a positive value
* Return Value  : The over-speed error flag
***********************************************************************************************************************/
static inline uint16_t mtr_check_over_speed_error(float f4_speed_rad, float f4_overspeed_limit_rad)
{
    float f4_temp0;
    uint16_t u2_temp0;

    u2_temp0 = MTR_ERROR_NONE;

    f4_temp0 = fabsf(f4_speed_rad);
    if (f4_temp0 > f4_overspeed_limit_rad)
    {
        u2_temp0 = MTR_ERROR_OVER_SPEED;                             /* Over speed error */
    }

    return (u2_temp0);
} /* End of function mtr_check_over_speed_error */

/***********************************************************************************************************************
* Function Name : mtr_check_over_voltage_error
* Description   : Checks over-voltage error
* Arguments     : f4_vdc - The actual Vdc value [V]
                  f4_overvoltage_limit - The threshold voltage[V] of over-voltage error
* Return Value  : The flag of over-voltage error
***********************************************************************************************************************/
static inline uint16_t mtr_check_over_voltage_error(float f4_vdc, float f4_overvoltage_limit)
{
    uint16_t u2_temp0;

    u2_temp0 = MTR_ERROR_NONE;

    if (f4_vdc > f4_overvoltage_limit)
    {
        u2_temp0 = MTR_ERROR_OVER_VOLTAGE;                           /* Over voltage error */
    }

    return (u2_temp0);
} /* End of function mtr_check_over_voltage_error */

/***********************************************************************************************************************
* Function Name : mtr_check_under_voltage_error
* Description   : Checks under-voltage error
* Arguments     : f4_vdc - The actual Vdc value [V]
                  f4_undervoltage_limit - The threshold voltage[V] of under-voltage error
* Return Value  : The flag of under-voltage error
***********************************************************************************************************************/
static inline uint16_t mtr_check_under_voltage_error(float f4_vdc, float f4_undervoltage_limit)
{
    uint16_t u2_temp0;

    u2_temp0 = MTR_ERROR_NONE;

    if (f4_vdc < f4_undervoltage_limit)
    {
        u2_temp0 = MTR_ERROR_UNDER_VOLTAGE;                          /* Under voltage error */
    }

    return (u2_temp0);
} /* End of function mtr_check_under_voltage_error */

/***********************************************************************************************************************
* Function Name : mtr_check_over_current_error
* Description   : Checks over-current error
* Arguments     : f4_iu - The actual U-phase current value [A]
*                 f4_iv - The actual V-phase current value [A]
*                 f4_iw - The actual W-phase current value [A]
*                 f4_overcurrent_limit - The threshold current[A] of over-current error
* Return Value  : The flag of over-current error
***********************************************************************************************************************/
static inline uint16_t mtr_check_over_current_error(float f4_iu, float f4_iv, float f4_iw, float f4_overcurrent_limit)
{
    float f4_temp0;
    uint16_t u2_temp0;

    u2_temp0 = MTR_ERROR_NONE;                                       /* Initialize */

    f4_temp0 = fabsf(f4_iu);
    u2_temp0 |= (f4_temp0 > f4_overcurrent_limit);
    f4_temp0 = fabsf(f4_iv);
    u2_temp0 |= (f4_temp0 > f4_overcurrent_limit);
    f4_temp0 = fabsf(f4_iw);
    u2_temp0 |= (f4_temp0 > f4_overcurrent_limit);

    u2_temp0 *= MTR_ERROR_OVER_CURRENT_SW;

    return (u2_temp0);
} /* End of function mtr_check_over_current_error */

/***********************************************************************************************************************
* Function Name : mtr_error_check
* Description   : Checks the errors
* Arguments     : st_foc - The pointer to the FOC data structure
* Return Value  : None
***********************************************************************************************************************/
void mtr_error_check(mtr_foc_control_t *st_foc)
{
    uint16_t u2_error_flags = 0;

    /*==================================*/
    /* Over current error check     */
    /*==================================*/
    u2_error_flags |= mtr_check_over_current_error(st_foc->f4_iu_ad,
                                                   st_foc->f4_iv_ad,
                                                   st_foc->f4_iw_ad,
                                                   st_foc->f4_overcurrent_limit);

    /*==================================*/
    /* Over voltage error check         */
    /*==================================*/
    u2_error_flags |= mtr_check_over_voltage_error(st_foc->f4_vdc_ad,
                                                   st_foc->f4_overvoltage_limit);

    /*===================================*/
    /* Under voltage error check         */
    /*===================================*/
    if (MTR_FLG_SET == st_foc->u1_flag_charge_cap)    /* Check "capacitor charge completed" */
    {
        u2_error_flags |= mtr_check_under_voltage_error(st_foc->f4_vdc_ad, st_foc->f4_undervoltage_limit);
    }

    /*================================*/
    /* Over speed error check     */
    /*================================*/
    u2_error_flags |= mtr_check_over_speed_error(st_foc->st_sc.f4_speed_rad, st_foc->f4_overspeed_limit_rad);

    if (MTR_ERROR_NONE != u2_error_flags)
    {
        mtr_foc_error(st_foc, u2_error_flags);              /* Execute error event */
    }
} /* End of function mtr_error_check */

/***********************************************************************************************************************
* Function Name : mtr_over_current
* Description   : POE interrupt
* Arguments     : st_foc - The pointer to the FOC data structure
* Return Value  : None
***********************************************************************************************************************/
void mtr_over_current(mtr_foc_control_t *st_foc)
{
    mtr_foc_error(st_foc, MTR_ERROR_OVER_CURRENT_HW);

    mtr_clear_oc_flag();                              /* Clear forced cutoff flag (detect over current) */
} /* End of function mtr_over_current */

/***********************************************************************************************************************
* Function Name : mtr_hall_error
* Description   : Hall angle error process
* Arguments     : st_foc - The pointer to the FOC data structure
*                 f4_hall_angle_rad - Rotor angle of signal detected for hall sensor
* Return Value  : None
***********************************************************************************************************************/
void mtr_hall_error(mtr_foc_control_t *st_foc, float f4_hall_angle_rad)
{
    /* hall error check */
    if(MTR_HALL_EDGE_ERROR == f4_hall_angle_rad)
    {
        mtr_statemachine_event(&st_foc->st_stm, st_foc, MTR_EVENT_ERROR);
        st_foc->u2_error_status |= MTR_ERROR_HALL_PATTERN;
                                                     /* error status update : HALL_PATTERN error */
    }
} /* End of function mtr_hall_error */

/***********************************************************************************************************************
* Function Name : mtr_foc_voltage_limit
* Description   : Limit voltage vector, d-axis voltage has higher priority than q-axis voltage
* Arguments     : p_foc - The pointer to the FOC structure
* Return Value  : None
***********************************************************************************************************************/
void mtr_foc_voltage_limit(mtr_foc_control_t *p_foc)
{
    float f4_va_max;
    float f4_vq_lim;
    float f4_vd_ref;

    f4_va_max = R_MOD_GetVamax(&p_foc->st_mod);
    f4_vd_ref = p_foc->st_cc.f4_vd_ref;

    /* The d-axis voltage has high priority than the q-axis */
    /* If absolute value of D-axis voltage exceeds maximum voltage, limit it to the maximum voltage */
    if (f4_vd_ref > f4_va_max)
    {
        p_foc->st_cc.f4_vd_ref = f4_va_max;
        p_foc->st_cc.f4_vq_ref = 0.0f;
    }
    else if (f4_vd_ref < (-f4_va_max))
    {
        p_foc->st_cc.f4_vd_ref = -f4_va_max;
        p_foc->st_cc.f4_vq_ref = 0.0f;
    }
    else
    {
        f4_vq_lim = sqrtf((f4_va_max * f4_va_max) - (f4_vd_ref *  f4_vd_ref));
        p_foc->st_cc.f4_vq_ref = R_MTR_LimitfAbs(p_foc->st_cc.f4_vq_ref, f4_vq_lim);
    }
} /* End of function mtr_foc_voltage_limit */

/***********************************************************************************************************************
* Function Name : mtr_foc_set_nominal_current
* Description   : Setup current related limit values by the nominal current of motor
* Arguments     : st_cc - The pointer to the FOC current control structure
*                 f4_nominal_current_rms - The nominal current[Arms] of motor
* Return Value  : None
***********************************************************************************************************************/
void mtr_foc_set_nominal_current(mtr_foc_control_t *st_foc, float f4_nominal_current_rms)
{
    float f4_ia_max;

    f4_ia_max = f4_nominal_current_rms * MTR_SQRT_3;
    st_foc->f4_overcurrent_limit = (f4_nominal_current_rms * MTR_SQRT_2) * MTR_OVERCURRENT_MARGIN_MULT;
    if(st_foc->f4_overcurrent_limit > IP_CURRENT_LIMIT)
    {
        st_foc->f4_overcurrent_limit = IP_CURRENT_LIMIT;
    }
    st_foc->st_sc.st_pi_speed.f4_ilimit = f4_ia_max;
    st_foc->st_cc.f4_lim_iq = f4_ia_max;

} /* End of function mtr_foc_set_nominal_current */
