/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products. No
* other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED. TO THE MAXIMUM
* EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES
* SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS
* SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability of
* this software. By using this software, you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2017 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/
/***********************************************************************************************************************
* File Name   : r_mtr_driver_access.c
* Description : The processing of accessing driver
***********************************************************************************************************************/
/**********************************************************************************************************************
* History : DD.MM.YYYY Version
*         : 05.04.2017 1.00
*         : 07.07.2017 1.01
***********************************************************************************************************************/

/***********************************************************************************************************************
* Includes <System Includes> , "Project Includes"
***********************************************************************************************************************/
/* Standard library headers */
#include <stdint.h>
#include <math.h>

/* Main associated header file */
#include "r_mtr_driver_access.h"

/* Project headers */
#include "r_mtr_ics.h"
#include "r_mtr_common.h"
#include "r_mtr_statemachine.h"
#include "r_mtr_parameter.h"
#include "r_mtr_foc_speed.h"
#include "r_mtr_ctrl_mcu.h"
#include "r_mtr_ctrl_mrssk.h"
#include "r_mtr_ctrl_hall.h"
#include "r_mtr_ctrl_encoder.h"
#include "r_mtr_foc_control_encd_position.h"

/***********************************************************************************************************************
* Private function declarations
***********************************************************************************************************************/
static inline mtr_foc_control_t* get_foc_by_id(uint8_t u1_id);

/***********************************************************************************************************************
* Private variables
***********************************************************************************************************************/
static mtr_ctrl_input_t st_ctrl_input_buff;     /* Structure for ICS input */
static uint8_t          u1_trig_enable_write;   /* The trigger for FOC command and configuration update */

/***********************************************************************************************************************
* Exported global variables
***********************************************************************************************************************/
extern mtr_foc_control_t g_st_foc;

/***********************************************************************************************************************
* Function Name : R_MTR_SetUserifMode
* Description   : Set User interface mode
* Arguments     : u1_user_mode - User interface mode status
* Return Value  : None
***********************************************************************************************************************/
void R_MTR_SetUserifMode(uint8_t u1_user_mode)
{
    st_ctrl_input_buff.u1_user_mode = u1_user_mode;
} /* End of function R_MTR_SetUserifMode */

/***********************************************************************************************************************
* Function Name : R_MTR_InitControl
* Description   : Set motor ID and INACTIVE IVENT
* Arguments     : u1_id - The motor ID which specify the motor connectors
* Return Value  : None
***********************************************************************************************************************/
void R_MTR_InitControl(uint8_t u1_id)
{
    mtr_foc_control_t *p_foc;

    p_foc = get_foc_by_id(u1_id);
    mtr_foc_init(p_foc, u1_id);       /* initialize foc */
} /* End of function R_MTR_InitControl */

/***********************************************************************************************************************
* Function Name : R_MTR_ExecEvent
* Description   : Excecution Event
* Arguments     : u1_event - Event
                  u1_id - The motor ID which specify the motor connectors
* Return Value  : None
***********************************************************************************************************************/
void R_MTR_ExecEvent(uint8_t u1_event, uint8_t u1_id)
{
    mtr_foc_control_t *p_foc;

    p_foc = get_foc_by_id(u1_id);
    mtr_statemachine_event(&p_foc->st_stm, p_foc, u1_event);
} /* End of function R_MTR_ExecEvent */

/***********************************************************************************************************************
* Function Name : R_MTR_ChargeCapacitor
* Description   : Wait for charging capacitor
* Arguments     : u1_id - The motor ID which specify the motor connectors
* Return Value  : None
***********************************************************************************************************************/
void R_MTR_ChargeCapacitor(uint8_t u1_id)
{
    mtr_foc_control_t *p_foc;

    p_foc = get_foc_by_id(u1_id);
    while (MTR_MCU_ON_V > p_foc->f4_vdc_ad)       /* wait until Vdc exceed MTR_MCU_ON_V */
    {
        p_foc->f4_vdc_ad = mtr_get_vdc(u1_id);
        R_MTR_ClearWdt();                          /* watch dog timer clear */
    }
    p_foc->u1_flag_charge_cap = MTR_FLG_SET;      /* flag set "capacitor charge completed" */
} /* End of function R_MTR_ChargeCapacitor */

/***********************************************************************************************************************
* Function Name : R_MTR_GetLoopModeStatus
* Description   : Get loop mode status
* Arguments     : u1_id - The motor ID which specify the motor connectors
* Return Value  : Control loop mode status
***********************************************************************************************************************/
uint8_t R_MTR_GetLoopModeStatus(uint8_t u1_id)
{
    mtr_foc_control_t *p_foc;

    p_foc = get_foc_by_id(u1_id);
    return (p_foc->u1_ctrl_loop_mode);
} /* End of function R_MTR_GetLoopModeStatus */

/***********************************************************************************************************************
* Function Name : R_MTR_SetPositionStatus
* Description   : Set position status
* Arguments     : position status
* Return Value  : None
***********************************************************************************************************************/
void R_MTR_SetPositionStatus(uint8_t u1_pos_status)
{
    st_ctrl_input_buff.u1_state_pos_ref = u1_pos_status;
} /* End of function R_MTR_SetPositionStatus */

/***********************************************************************************************************************
* Function Name : R_MTR_SetPosition
* Description   : Set position reference
* Arguments     : position reference [degree]
* Return Value  : None
***********************************************************************************************************************/
void R_MTR_SetPosition(int16_t s2_ref_position)
{
    st_ctrl_input_buff.s2_ref_position_deg = s2_ref_position;
} /* End of function R_MTR_SetPosition */

/***********************************************************************************************************************
* Function Name : R_MTR_GetPosition
* Description   : Get position [degree]
* Arguments     : u1_id - The motor ID which specify the motor connectors
* Return Value  : position [degree]
***********************************************************************************************************************/
int16_t R_MTR_GetPosition(uint8_t u1_id)
{
    int16_t s2_position_deg;
    mtr_foc_control_t *p_foc;

    p_foc = get_foc_by_id(u1_id);
    s2_position_deg = (int16_t)(p_foc->st_pc.f4_pos_rad * 360.0f) / MTR_TWOPI;

    return (s2_position_deg);
} /* End of function R_MTR_GetPosition */

/***********************************************************************************************************************
* Function Name : R_MTR_GetPositioningFlag
* Description   : Get Positioning Flag
* Arguments     : u1_id - The motor ID which specify the motor connectors
* Return Value  : positioning flag
***********************************************************************************************************************/
uint8_t R_MTR_GetPositioningFlag(uint8_t u1_id)
{
    uint8_t u1_temp;
    float f4_temp;
    mtr_foc_control_t *p_foc;

    p_foc = get_foc_by_id(u1_id);
    if ((MTR_MODE_DRIVE == p_foc->u2_run_mode) && (MTR_LOOP_POSITION == p_foc->u1_ctrl_loop_mode))
    {
        /* Completed Positioning Flag */
        f4_temp = fabsf(p_foc->st_pc.f4_pos_err_rad);
        if (p_foc->st_pc.u2_pos_band_limit * p_foc->st_ec.f4_encd_angle_diff >= f4_temp)
        {
            u1_temp = MTR_FLG_SET;
        }
        else
        {
            u1_temp = MTR_FLG_CLR;
        }
    }
    else
    {
        u1_temp = MTR_FLG_CLR;
    }

    return (u1_temp);
} /* End of function R_MTR_GetPositioningFlag */

/***********************************************************************************************************************
* Function Name : R_MTR_SetSpeed
* Description   : Set speed reference
* Arguments     : ref_speed - speed reference [rpm]
* Return Value  : None
***********************************************************************************************************************/
void R_MTR_SetSpeed(int16_t ref_speed)
{
    st_ctrl_input_buff.s2_ref_speed_rpm = ref_speed;
} /* End of function R_MTR_SetSpeed */

/***********************************************************************************************************************
* Function Name : R_MTR_GetSpeed
* Description   : Get speed value
* Arguments     : u1_id - The motor ID which specify the motor connectors
* Return Value  : speed
***********************************************************************************************************************/
int16_t R_MTR_GetSpeed(uint8_t u1_id)
{
    int16_t s2_speed_rpm;
    mtr_foc_control_t *p_foc;

    p_foc = get_foc_by_id(u1_id);
    s2_speed_rpm = (int16_t)(p_foc->st_sc.f4_speed_rad / p_foc->st_sc.f4_rpm_rad);

    return (s2_speed_rpm);
} /* End of function R_MTR_GetSpeed */

/***********************************************************************************************************************
* Function Name : R_MTR_SetDir
* Description   : Set rotation direction
* Arguments     : dir - rotational direction
*                 u1_id - The motor ID which specify the motor connectors
* Return Value  : None
***********************************************************************************************************************/
void R_MTR_SetDir(uint8_t dir, uint8_t u1_id)
{
    mtr_foc_control_t *p_foc;

    p_foc = get_foc_by_id(u1_id);
    p_foc->u1_direction = dir;
} /* End of function R_MTR_SetDir */

/***********************************************************************************************************************
* Function Name : R_MTR_GetDir
* Description   : Get rotation direction
* Arguments     : u1_id - The motor ID which specify the motor connectors
* Return Value  : rotational direction
***********************************************************************************************************************/
uint8_t R_MTR_GetDir(uint8_t u1_id)
{
    uint8_t u1_direction;
    mtr_foc_control_t *p_foc;

    p_foc = get_foc_by_id(u1_id);
    u1_direction = p_foc->u1_direction;

    return (u1_direction);
} /* End of function R_MTR_GetDir */

/***********************************************************************************************************************
* Function Name : R_MTR_GetStatus
* Description   : Get status of motor control system
* Arguments     : u1_id - The motor ID which specify the motor connectors
* Return Value  : motor status
***********************************************************************************************************************/
uint8_t R_MTR_GetStatus(uint8_t u1_id)
{
    uint8_t u1_mode_system;
    mtr_foc_control_t *p_foc;

    p_foc = get_foc_by_id(u1_id);
    u1_mode_system = mtr_statemachine_get_status(&p_foc->st_stm);

    return (u1_mode_system);
} /* End of function R_MTR_GetStatus */

/***********************************************************************************************************************
* Function Name : R_MTR_InputBuffParamReset
* Description   : Reset input buffer variables when motor control reset
* Arguments     : None
* Return Value  : None
***********************************************************************************************************************/
void R_MTR_InputBuffParamReset(void)
{
    /* initialize member of ICS input buffer structure */
    st_ctrl_input_buff.u1_user_mode             = MTR_DISABLE_AUTO_SET;
    st_ctrl_input_buff.u1_direction             = MTR_CW;
    st_ctrl_input_buff.u1_ctrl_loop_mode_buff   = LOOP_MODE;
    st_ctrl_input_buff.u1_ctrl_method_mode      = POS_CTRL_MODE;
    st_ctrl_input_buff.u1_state_pos_ref         = MTR_POS_TRAPEZOID;
    st_ctrl_input_buff.u1_encd_angle_adj_mode   = ANGLE_ADJUST_MODE;
    st_ctrl_input_buff.s2_ref_position_deg      = 0;
    st_ctrl_input_buff.s2_ref_speed_rpm         = 0;
    st_ctrl_input_buff.u2_min_speed_rpm         = MTR_MIN_SPEED_RPM;
    st_ctrl_input_buff.u2_max_speed_rpm         = MTR_MAX_SPEED_RPM;
    st_ctrl_input_buff.u2_speed_limit_rpm       = MTR_SPEED_LIMIT_RPM;
    st_ctrl_input_buff.u2_hs_change_speed_rpm   = MTR_HS_CHANGE_RPM;
    st_ctrl_input_buff.u2_hs_change_margin_rpm  = MTR_HS_CHANGE_MARGIN_RPM;
    st_ctrl_input_buff.u2_offset_calc_time      = MTR_OFFSET_CALC_TIME;
    st_ctrl_input_buff.u2_pos_dead_band         = MTR_POS_DEAD_BAND;
    st_ctrl_input_buff.u2_pos_band_limit        = MTR_POS_BAND_LIMIT;
    st_ctrl_input_buff.u2_pos_interval_time     = MTR_POS_INTERVAL_TIME;
    st_ctrl_input_buff.u2_encd_cpr_mech         = MTR_ENCD_CPR_MECH;
    st_ctrl_input_buff.f4_accel_time            = MTR_ACCEL_TIME;
    st_ctrl_input_buff.f4_ol_ref_id             = MTR_REF_ID;
    st_ctrl_input_buff.f4_id_up_time            = MTR_ID_UP_TIME;
    st_ctrl_input_buff.f4_limit_speed_change    = MTR_LIMIT_SPEED_CHANGE;
    st_ctrl_input_buff.f4_ipd_pos_kp_ratio      = POS_KP_RATIO;
    st_ctrl_input_buff.f4_ipd_speed_k_ratio     = POS_SPEED_RATIO;
    st_ctrl_input_buff.f4_ipd_err_limit_1       = POS_IPD_ERR_LIMIT_1;
    st_ctrl_input_buff.f4_ipd_err_limit_2       = POS_IPD_ERR_LIMIT_2;

    /* motor parameter */
    st_ctrl_input_buff.st_motor.u2_mtr_pp       = MTR_POLE_PAIRS;
    st_ctrl_input_buff.st_motor.f4_mtr_r        = MTR_R;
    st_ctrl_input_buff.st_motor.f4_mtr_ld       = MTR_LD;
    st_ctrl_input_buff.st_motor.f4_mtr_lq       = MTR_LQ;
    st_ctrl_input_buff.st_motor.f4_mtr_m        = MTR_M;
    st_ctrl_input_buff.st_motor.f4_mtr_j        = MTR_J;
    st_ctrl_input_buff.st_motor.f4_nominal_current_rms = MTR_NOMINAL_CURRENT_RMS;

    /* design parameter */
    st_ctrl_input_buff.st_design_params.f4_current_omega = MTR_CURRENT_OMEGA;
    st_ctrl_input_buff.st_design_params.f4_current_zeta  = MTR_CURRENT_ZETA;
    st_ctrl_input_buff.st_design_params.f4_speed_omega   = MTR_SPEED_OMEGA;
    st_ctrl_input_buff.st_design_params.f4_speed_zeta    = MTR_SPEED_ZETA;
    st_ctrl_input_buff.st_design_params.f4_pos_omega     = MTR_POS_OMEGA;
    st_ctrl_input_buff.st_design_params.f4_sob_omega     = SOB_OMEGA;
    st_ctrl_input_buff.st_design_params.f4_sob_zeta      = SOB_ZETA;

    /* pi gain */
    st_ctrl_input_buff.st_ctrl_gain.f4_id_kp    = 0;
    st_ctrl_input_buff.st_ctrl_gain.f4_id_ki    = 0;
    st_ctrl_input_buff.st_ctrl_gain.f4_iq_kp    = 0;
    st_ctrl_input_buff.st_ctrl_gain.f4_iq_ki    = 0;
    st_ctrl_input_buff.st_ctrl_gain.f4_speed_kp = 0;
    st_ctrl_input_buff.st_ctrl_gain.f4_speed_ki = 0;
    st_ctrl_input_buff.st_ctrl_gain.f4_pos_kp   = 0;

    /* initialize control gains */
    R_MTR_CtrlGainCalc();

    /* initialize write enable switch */
    u1_trig_enable_write = 0;
} /* End of function R_MTR_InputBuffParamReset */

/***********************************************************************************************************************
* Function Name : R_MTR_CtrlInput
* Description   : Store com_* variables to control input buffer
* Arguments     : Control input structure (pointer)
* Return Value  : None
***********************************************************************************************************************/
void R_MTR_CtrlInput(mtr_ctrl_input_t *st_ctrl_input)
{
    /* transfer control input value to buffer structure */
    st_ctrl_input_buff = *st_ctrl_input;
    /* initialize write enable switch */
    u1_trig_enable_write = MTR_FLG_SET;
} /* End of function R_MTR_CtrlInput */

/***********************************************************************************************************************
* Function Name : R_MTR_SetVariables
* Description   : Set control input buffer to motor control structure members
* Arguments     : None
* Return Value  : None
***********************************************************************************************************************/
void R_MTR_SetVariables(void)
{
    /**********************/
    /*** FOC structuire ***/
    /**********************/
    /* Setup motor parameter */
    g_st_foc.st_motor = st_ctrl_input_buff.st_motor;
    /* Setup position command value management */
    g_st_foc.u1_state_pos_ref = st_ctrl_input_buff.u1_state_pos_ref;
    /* Setup encoder angle adjustment mode */
    g_st_foc.u1_encd_angle_adj_mode = st_ctrl_input_buff.u1_encd_angle_adj_mode;
    /* Setup current offset calculation time */
    g_st_foc.u2_offset_calc_time = st_ctrl_input_buff.u2_offset_calc_time;
    /* Addition value of id reference when doing encoder angle alignment */
    g_st_foc.f4_id_up_step = g_st_foc.f4_ol_ref_id / st_ctrl_input_buff.f4_id_up_time;
    /* Setup max speed limit for encoder singal input filter */
    g_st_foc.f4_max_speed_rc_fil_rpm = 
                60.0f / (MTR_RC_FIL_TIME_CONS * MTR_RC_FIL_TIME_CONS_MAG * st_ctrl_input_buff.u2_encd_cpr_mech);
    /* Setup nominal current */
    mtr_foc_set_nominal_current(&g_st_foc, st_ctrl_input_buff.st_motor.f4_nominal_current_rms);

    /********************************/
    /*** Speed control structuire ***/
    /********************************/
    /* Setup rotation direction */
    mtr_set_param_ref_dir(&g_st_foc.st_sc,st_ctrl_input_buff.u1_direction);
    /* [rpm]->[rad/s]*/
    mtr_set_param_rpm_to_rad(&g_st_foc.st_sc, g_st_foc.st_motor.u2_mtr_pp);
    /* Setup reference speed */
    mtr_set_param_ref_speed(&g_st_foc.st_sc, st_ctrl_input_buff.s2_ref_speed_rpm);
    /* Setup max speed */
    if(st_ctrl_input_buff.u2_max_speed_rpm > (uint16_t)g_st_foc.f4_max_speed_rc_fil_rpm)
    {
        st_ctrl_input_buff.u2_max_speed_rpm = (uint16_t)g_st_foc.f4_max_speed_rc_fil_rpm;
    }
    mtr_set_param_max_speed(&g_st_foc.st_sc, st_ctrl_input_buff.u2_max_speed_rpm);

    /* Setup limit of speed change */
    mtr_set_param_limit_speed_change(&g_st_foc.st_sc, st_ctrl_input_buff.f4_limit_speed_change);
    /* Setup over-speed limit */
    g_st_foc.f4_overspeed_limit_rad = (float)(st_ctrl_input_buff.u2_speed_limit_rpm * g_st_foc.st_sc.f4_rpm_rad);

    /***********************************/
    /*** Position control structuire ***/
    /***********************************/
    /* Setup control loop select */
    if (MTR_MODE_INACTIVE == mtr_statemachine_get_status(&g_st_foc.st_stm))
    {
        g_st_foc.u1_ctrl_loop_mode = st_ctrl_input_buff.u1_ctrl_loop_mode_buff;
        if (MTR_LOOP_SPEED == g_st_foc.u1_ctrl_loop_mode)
        {
            g_st_foc.u1_state_pos_ref = MTR_POS_CONST;
        }
    }
    /* Setup control method select */
    if (MTR_LOOP_SPEED == g_st_foc.u1_ctrl_loop_mode)
    {
        g_st_foc.u1_ctrl_method_mode = MTR_CTRL_PID;
    }
    else if (MTR_LOOP_POSITION == g_st_foc.u1_ctrl_loop_mode)
    {
        g_st_foc.u1_ctrl_method_mode = st_ctrl_input_buff.u1_ctrl_method_mode;
    }
    else
    {
        /* Do Nothing */
    }
    /* Setup motor position dead-band */
    mtr_set_param_pos_dead_band(&g_st_foc.st_pc, st_ctrl_input_buff.u2_pos_dead_band);
    /* Setup positioning band limit */
    mtr_set_param_pos_band_limit(&g_st_foc.st_pc, st_ctrl_input_buff.u2_pos_band_limit);
    /* Setup reference position */
    mtr_set_param_reference_position(&g_st_foc.st_pc, st_ctrl_input_buff.s2_ref_position_deg);

    /**********************************/
    /*** Encoder control structuire ***/
    /**********************************/
    /* Setup encoder counts per revolution (mechanical) */
    /* Setup inverse number of encoder counts per revolution (mechanical) */
    /* Setup angle of encoder 1 pulse width */
    mtr_set_param_encd_cpr_mech(&g_st_foc.st_ec, st_ctrl_input_buff.u2_encd_cpr_mech);
    /* Setup toggle speed for high speed mode */
    mtr_set_param_encd_hs_sw_speed_rad(&g_st_foc.st_ehc,
                                       st_ctrl_input_buff.u2_hs_change_speed_rpm,
                                       g_st_foc.st_sc.f4_rpm_rad);
    mtr_set_param_encd_hs_sw_speed_margin_rad(&g_st_foc.st_ehc,
                                              st_ctrl_input_buff.u2_hs_change_margin_rpm,
                                              g_st_foc.st_sc.f4_rpm_rad);

    /*************************************/
    /*** Position profiling structuire ***/
    /*************************************/
    /* Setup interval time for reference position update */
    mtr_set_param_interval_time(&g_st_foc.st_ppf, st_ctrl_input_buff.u2_pos_interval_time);
    /* Setup acceleration time */
    mtr_set_param_accel_time(&g_st_foc.st_ppf, st_ctrl_input_buff.f4_accel_time);
    /* Setup reference max speed */
    mtr_set_param_accel_max_speed(&g_st_foc.st_ppf, st_ctrl_input_buff.u2_max_speed_rpm);
    /* Setup max acceleration time */
    mtr_set_param_max_accel_time(&g_st_foc.st_ppf,
                                 &st_ctrl_input_buff.st_motor);

    /******************************/
    /*** IPD control structuire ***/
    /******************************/
    /* Setup position control gain ratio for I-PD */
    mtr_set_param_pos_kp_ratio(&g_st_foc.st_ipd, st_ctrl_input_buff.f4_ipd_pos_kp_ratio);
    /* Setup speed gain ratio for I-PD */
    mtr_set_param_speed_k_ratio(&g_st_foc.st_ipd, st_ctrl_input_buff.f4_ipd_speed_k_ratio);
    /* Setup position error limit */
    mtr_set_param_err_limit_1(&g_st_foc.st_ipd, st_ctrl_input_buff.f4_ipd_err_limit_1);
    mtr_set_param_err_limit_2(&g_st_foc.st_ipd, st_ctrl_input_buff.f4_ipd_err_limit_2);
} /* End of function R_MTR_SetVariables */

/***********************************************************************************************************************
* Function Name : R_MTR_AutoSetVariables
* Description   : Set control input buffer to motor control structure members
* Arguments     : None
* Return Value  : None
***********************************************************************************************************************/
void R_MTR_AutoSetVariables(void)
{
    if(MTR_ENABLE_AUTO_SET == st_ctrl_input_buff.u1_user_mode)
    {
        /* position command value management */
        g_st_foc.u1_state_pos_ref = st_ctrl_input_buff.u1_state_pos_ref;
        if(MTR_LOOP_POSITION == st_ctrl_input_buff.u1_ctrl_loop_mode_buff)
        {
            /* reference position */
            mtr_set_param_reference_position(&g_st_foc.st_pc, st_ctrl_input_buff.s2_ref_position_deg);
        }
        else if(MTR_LOOP_SPEED == st_ctrl_input_buff.u1_ctrl_loop_mode_buff)
        {
            /* reference speed */
            mtr_set_param_ref_speed(&g_st_foc.st_sc, st_ctrl_input_buff.s2_ref_speed_rpm);
        }
    }
} /* End of function R_MTR_AutoSetVariables */

/***********************************************************************************************************************
* Function Name : R_MTR_CtrlGainCalc
* Description   : motor control parameter & gain calculation
* Arguments     : None
* Return Value  : None
***********************************************************************************************************************/
void R_MTR_CtrlGainCalc(void)
{
#if (GAIN_MODE == MTR_GAIN_DESIGN_MODE)
    /* d-axis q-axis current PI gain */
    mtr_foc_current_pi_gain_calc(&g_st_foc.st_motor,
                                 &st_ctrl_input_buff.st_design_params,
                                 &g_st_foc.st_cc, g_st_foc.st_cc.f4_ctrl_period);
    /* speed PI gain */
    mtr_speed_pi_gain_calc(&g_st_foc.st_motor,
                           &st_ctrl_input_buff.st_design_params,
                           &g_st_foc.st_sc, g_st_foc.st_sc.f4_speed_ctrl_period);
    /* position P gain */
    mtr_pos_p_gain_calc(&st_ctrl_input_buff.st_design_params, &g_st_foc.st_pc);
#elif (GAIN_MODE == MTR_GAIN_DIRECT_MODE)
    /* pi gain */
    mtr_pi_ctrl_set_kp(&g_st_foc.st_cc.st_pi_id, st_ctrl_input_buff.st_ctrl_gain.f4_id_kp);
    mtr_pi_ctrl_set_ki(&g_st_foc.st_cc.st_pi_id, st_ctrl_input_buff.st_ctrl_gain.f4_id_ki);
    mtr_pi_ctrl_set_kp(&g_st_foc.st_cc.st_pi_iq, st_ctrl_input_buff.st_ctrl_gain.f4_iq_kp);
    mtr_pi_ctrl_set_ki(&g_st_foc.st_cc.st_pi_iq, st_ctrl_input_buff.st_ctrl_gain.f4_iq_ki);
    mtr_pi_ctrl_set_kp(&g_st_foc.st_sc.st_pi_speed, st_ctrl_input_buff.st_ctrl_gain.f4_speed_kp);
    mtr_pi_ctrl_set_ki(&g_st_foc.st_sc.st_pi_speed, st_ctrl_input_buff.st_ctrl_gain.f4_speed_ki); 
    mtr_set_param_position_kp(&g_st_foc.st_pc, st_ctrl_input_buff.st_ctrl_gain.f4_pos_kp);
#endif
    /* I-PD control gain */
    mtr_ipd_ctrl_gain_calc(&g_st_foc.st_motor,
                           &st_ctrl_input_buff.st_design_params,
                           &g_st_foc.st_ipd, g_st_foc.st_sc.f4_speed_ctrl_period);
    /* speed observer */
    mtr_speed_observer_gain_calc(&g_st_foc.st_sob,
                                 &st_ctrl_input_buff.st_design_params,
                                 g_st_foc.st_sc.f4_speed_ctrl_period);
} /* End of function R_MTR_CtrlGainCalc */

/***********************************************************************************************************************
* Function Name : R_MTR_UpdatePolling
* Description   : Polling u1_trig_enable_write to update configurations and commands
* Arguments     : None
* Return Value  : None
***********************************************************************************************************************/
void R_MTR_UpdatePolling(void)
{
    if (MTR_FLG_SET == u1_trig_enable_write)
    {
        R_MTR_SetVariables();
        R_MTR_CtrlGainCalc();
        u1_trig_enable_write = MTR_FLG_CLR;
    }
} /* End of function R_MTR_UpdatePolling */

/***********************************************************************************************************************
* Function Name : get_foc_by_id
* Description   : Gets the pointer of FOC structure corresponding to the given motor ID
* Arguments     : u1_id - The motor ID which specify the motor connectors
* Return Value  : Foc control structure pointer
***********************************************************************************************************************/
static inline mtr_foc_control_t* get_foc_by_id(uint8_t u1_id)
{
    switch (u1_id)
    {
        case MTR_ID_A:
            return (&g_st_foc);
        break;

        default:
            return 0;
        break;
    }
} /* End of function get_foc_by_id */
